
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
//; C Runtime Startup Code for a cybiko boot application - crt0 - C RunTime 0
//; The C runtime carries out the following actions:
//;    * Initialize the stacks.
//;    * Copy the contents of the .data (initialized data) section from
//;       non-volatile memory should it be required.
//;    * Copy the contents of the .fast section from non-volatile memory to SRAM
//;       should it be required.
//;    * Initialize the .bss section.
//;    * Initialize the heap.
//;    * Call constructors.
//;    * Jump to the main entry point.
//;    * Call destructors.
//;    * Wait in exit loop.
//;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

.h8300s     ;generate H8S code
.text

end_of_ram = 0x23F000   ; leave off some room for the stack

.long       0x332211ff  ;cyborn apploader looks for this
.long       _start      ;start location entry point - ROM uses this for entry

.global     _start      ;the linker is looking for this symbol


;By the time we get here, the stack is set up and the display has been
;initialized, a timer is started at 50Hz thanks to the monitor/bootloader in ROM
;rs-232 is setup and waiting for this boot program!

_start:
    ;zero the bss segment
    sub     e1,             e1      ;clear e1 to zero
    mov     #_bss_start,    er2     ;where to start from
zerobcc:
    cmp     #_end,          er2     ;check for the end of segment
    beq     zerodone
    mov     e1,             @er2    ;put the zero into the word
    adds    #0x2,           er2     ;point to next word
    bra     zerobcc

zerodone:


    ; jump to the entry point for the C program
    jsr     _main                   ;start the program


    ;TODO should have some kind of atexit area to support atexit


    ;cleanup for when the program exits
    ; jump back to the beginning - where the reset vector points to ROM start
    jmp     @@0                     ;system reset


