/* For building a .boot executable that is sent to the cybiko during boot */
/* The classic cybiko uses a H8S/2241 processor in advanced mode (6) */

OUTPUT_FORMAT("coff-h8300")
/*OUTPUT_FORMAT("elf32-h8300")*/
OUTPUT_ARCH(h8300s)
ENTRY("_start")

MEMORY
{
    /* 0xc4 is a magic entry.  We should have the linker just
       skip over it one day...  */
    vectors : o = 0x0000, l = 0x16f
    /* what are magicvectors? for DTC? */
    magicvectors : o = 0x16f, l = 0x16f
    /* Cybiko Classic has 256K ram starting at 0x200000 */
    ram    : o = 0x200000, l = 0x40000
    /* The stack starts at the top of on-chip-RAM. Says me! */
    /* The stack goes backwards from hi to lo mem */
    topram : o = 0xfffbfc, l = 0x4
    /* This holds variables in the "tiny" sections. 4K - 0x200 - 0x200 */
    tiny   : o = 0xffea00, l = 0x0b00
    /* Address space for the 8-bit area. Use this for? */
    eight  : o = 0xfff800, l = 0x0200
}

SECTIONS
{
.vectors :
	{
	  /* Use something like this to place a specific
	     function's address into the vector table.
	     LONG (ABSOLUTE (_foobar)).  */
	  *(.vectors)
	}  > vectors
.text :
    {
        *(.text)
        *(.rodata)
        *(.strings)
           _etext = . ;
    }  > ram
.tors :
	{
	  ___ctors = . ;
	  *(.ctors)
	  ___ctors_end = . ;
	  ___dtors = . ;
	  *(.dtors)
	  ___dtors_end = . ;
	} > ram
.data :
    {
        *(.data)
        _edata = . ;
    }  > ram
.bss :
    {
        _bss_start = . ;
        *(.bss)
        *(COMMON)
        _end = . ;
    }  >ram
.stack :
    {
        _stack = . ;
        *(.stack)
    }  > topram
.tiny :
    {
        *(.tiny)
    }  > tiny
.eight :
    {
        *(.eight)
    }  > eight
.stab 0 (NOLOAD) :
    {
        [ .stab ]
    }
.stabstr 0 (NOLOAD) :
    {
        [ .stabstr ]
    }
}
