
# Makefile for the CyBorn OS for the classic Cybiko
# The filetype for the OS is binary - not ELF or coff


# This are the generic rules that help make cybiko programs
# NO TARGETS in this file - it cannot be 'made' just inlcuded like this in
# EVERY makefile for the cybiko:
# include "cyborn.mk"

# SHELL = /bin/sh

# some of the routines are located in ROM - a certain version - mine is 117
BOOTROM_VERSION = 117

# set the path to here
# really should set environment variable
#CYBORNPATH = v:/projects/cyborn


# make sure that the path for tools points to the GCC toolchain dir for H8S
TOOLDIR=$(CYBORNPATH)/tools/bin
#TOOLDIR=$(CYBORNPATH)/h8/bin

# COFF format for now - must change to ELF since COFF support is being dropped
TOOLPATH=$(TOOLDIR)/h8300-hms-
#TOOLPATH=$(TOOLDIR)/h8300-elf-


# object files to start then execute the OS - these are needed, add the others
BOOTPROGRAM = $(CYBORNPATH)/build/crt0boot.o
BOOTLAYOUT = $(CYBORNPATH)/build/cybikoboot.ld

# The FLASHLAYOUT will map .text into the flash area - this can be used
### for making an OS that will be put into flash
#FLASHPROGRAM = $(CYBORNPATH)/build/crt0flash.o
FLASHLAYOUT = $(CYBORNPATH)/build/cybikoflash.ld

# TODO - will need a start and layout for apps that will be read and relocated
#APPPROGRAM = $(CYBORNPATH)/build/crt0app.o
#APPLAYOUT = $(CYBORNPATH)/build/cybikoapp.ld


# library path that has all libraries in it. The compiler should use this to
# know where to look for libraries
LIBPATH = $(CYBORNPATH)/lib
LIBS = -L $(LIBPATH) -lcyborn -lgfx -lc


# ALL of the includes go in here - just cuz
INCLUDES = $(CYBORNPATH)/include



# Special macro definitions:
#   $@ is the name of the file to be made
#   $< is the name of the related file that caused the action



# generic building rules
%.o: %.c
	$(TOOLPATH)gcc.exe -I $(INCLUDES) -ms -O2 -Wall -fomit-frame-pointer -c -o $@ $<

#$(TOOLPATH)gcc.exe -I $(INCLUDES) -ms -O2 -Wall -ffreestanding -fomit-frame-pointer -c -o $@ $<

%.o: %.s
	$(TOOLPATH)as.exe  -defsym __H8300S__=1 -I $(INCLUDES) -o $@ $<

%.o: %.S
	$(TOOLPATH)gcc.exe -c -DBOOTROM117 -I $(INCLUDES) -o  $@ $<


# Linker Flags:    - this option can cause errors sometimes
LDFLAGS = --relax




