#ifndef _FONT_H_
#define _FONT_H_

// need this for bitmap and pixmap defines
#include "gfx.h"


#define FONT_SPACING_DEFAULT 0
#define FONT_LEADING_DEFAULT 1


// Fonts - includes the file format
// This is specific for cybiko because of the 2 bits for cybiko gray scale
// but it could be used for some other application

typedef enum
{
    FONT_TYPE_PROP = 0x80,  // proportional font
    FONT_TYPE_GRAY = 0x40,  // gray - uses pixmap instead of bitmap
    FONT_TYPE_FFIX = 0x20,  // forced to fixed - overrides prop
    FONT_TYPE_COMP = 0x10,  // compressed
} FontTypeT;


// This is the FILE format that all fonts for CyBorn must follow
// A conversion program can be run on .pdb, .fon, etc to make into this
//  format.
// TODO: there is somekind of bitmap conversion in the monitor ROM - I think
typedef struct
{
    char        name[20];
    FontTypeT   type;
    char        AsciiStart;
    char        AsciiEnd;
    char        HeightMax;
    char        WidthMax;   // contains all font width if not proportional
} FontInfoT;

// The rest of the file contains:
//    short       offset;    // offset of the width table
//    char        map;
//    char        widthsandoffsets;   // if proportional



// font structure that is used - hidden
// The structure will be implemented in the .c file
typedef struct font *FontT;

// use like this:
// FontT TestFont;
// TestFont = FontLoad(fontlocation);   // returns non-zero if OK
// if (TestFont == 0) return trouble;   // or some other error, etc
// width = FontWidth(TestFont);         // get width - max for proportional
// height = FontHeight(TestFont);       // get height
// pixels = FontWide(TestFont, str);    // how many pixels wide is string
// FontLeading(TestFont, 2);            // Set Leading to 2

typedef char    FontWidthT;
typedef char    FontHeightT;
typedef char    FontSpacingT;
typedef char    FontLeadingT;


// open the location and get the font
FontT
FontLoad(char *from);


// returns the maximum width for proportional, or the width of each
FontWidthT
FontWidth(FontT font);


// returns max height. TODO: decenders? etc...?
FontHeightT
FontHeight(FontT font);


// returns how many pixels wide the given string will be
unsigned
FontWide(FontT font, char *str);


// if the font is gray scale, the context color is ignored
unsigned
FontDisplay(GfxContextT *context, unsigned x, unsigned y, FontT font, char *str);


// set the spacing between each line of text - default = 1
void
FontLeading(FontT font, FontLeadingT leading);  // line leading (spacing)


// set the spacing between each character - default = 0
void
FontSpacing(FontT font, FontSpacingT spacing);  // char spacing


// Force a font to be fixed - all widths will be the max width for that font
// Has no effect on fixed fonts
void
FontFixed(FontT font);



#endif  // header
