#ifndef _LCD_H_
#define _LCD_H_


// Physical cybiko display LCD defines
#define LCD_ROWS    100
#define LCD_COLS    160

// These functions deal directly with the LCD.
//

// Using the LCD registers:
// First you must point to the register that you want to talk to by writing
// the index of the register to 0x600000. Then write the data for that register
// to 0x600001.
// The 0x600000/1 is where the LCD registers are mapped by the Cybiko H8S
#define LCD_INDEX(index)  (*(volatile char *)0x600000) = (index)
#define LCD_WRITE(data)   (*(volatile char *)0x600001) = (data)


#define HD66421_REG_R0    0x00 // Control Register 1
#define HD66421_REG_R1    0x01 // Control Register 2
#define HD66421_REG_R2    0x02 // X address Register
#define HD66421_REG_R3    0x03 // Y address Register
#define HD66421_REG_R4    0x04 // Display RAM access register
#define HD66421_REG_R5    0x05 // Display start line register
#define HD66421_REG_R6    0x06 // Blink start line register
#define HD66421_REG_R7    0x07 // Blink end line register
#define HD66421_REG_R8    0x08 // Blink register 1
#define HD66421_REG_R9    0x09 // Blink register 2
#define HD66421_REG_R10   0x0A // Blink register 3
#define HD66421_REG_R11   0x0B // Partial display block register
#define HD66421_REG_R12   0x0C // Gray scale pallete 1 (0,0)
#define HD66421_REG_R13   0x0D // Gray scale pallete 2 (0,1)
#define HD66421_REG_R14   0x0E // Gray scale pallete 3 (1,0)
#define HD66421_REG_R15   0x0F // Gray scale pallete 4 (1,1)
#define HD66421_REG_R16   0x10 // Contrast control register
#define HD66421_REG_R17   0x11 // Plane selection register


typedef struct
{
    unsigned row;
    unsigned col;
    unsigned width;
    unsigned height;
} LcdAreaT;



// Puts 'width' data to a row 'y' starting at 'x' column
// This has been optimized in a .S assembly file
inline void LcdRow(unsigned char *data, char col, char row, char width);


// Displays the 4000 bytes to the LCD
void LcdDisplay(unsigned char *data);


// Set the LCD area to a pattern / color
void LcdArea(const LcdAreaT *area, unsigned char pattern);


// clear/set the entire lcd to 'pattern'
void LcdClear(unsigned char pattern);


// Scroll the LCD - should be less than 100
void LcdScroll(unsigned char scroll);


// set the pallet for the LCD
void LcdPallet(char gray1, char gray2, char gray3, char gray4);


// Set the contrast
void LcdContrast(unsigned char contrast);




#endif

