//
//    Cyborn OS - Copyright (C) 2004 Steve Gotthardt - Gotthardt Designs
//
//    This file is part of the Cyborn distribution.
//
//    Cyborn is free software; you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation; either version 2 of the License, or
//    (at your option) any later version.
//
//    Cyborn is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Cyborn; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//    A special exception to the GPL can be applied should you wish to distribute
//    a combined work that includes Cyborn, without being obliged to provide
//    the source code for any proprietary components.
//




#ifndef _LOWLEVEL_H_
#define _LOWLEVEL_H_


// really we have a 2241
#include "2245s.h"
#include "types.h"


// from version.h
const char *RomVersion(void);



// bitmask(s)
// Some of these can be found in the bootrom
extern const UCHAR bitposition[8];



/* These are very low level drivers */


// vibrate - dont use these yet - the vibrate motor is not 
// protected from inducing high voltage into the driver xistor
//void _vibrate_on(void);
//void _vibrate_off(void);


///////////////////////////////////
// use one or the other for now
// sound
void _soundinit(void);
void _sound(unsigned raw);
void _soundoff(void);


// PWM sound
void _soundpwm(unsigned pwm);
void _soundinitpwm(void);
//////////////////////////////////



/////////////////////////////////////////////////////////////////////
// Wave API
/////////////////////////////////////////////////////////////////////
// Use:
//  WaveT gunshot;  // gunshot has been generated by utility and exported these:
//  gunshot.WaveBegin = &gunshot_begin;
//  gunshot.WaveEnd   = &gunshot_end;

//  _waveinit();
//
//  _waveplay(&gunshot_begin, &gunshot_end);
//
// optional:
// if (_wavedone() != 0)  notdoneyet();
//

typedef signed char *WaveT;

// Call this before using any other wave function
void _waveinit(void);

// Starts playing wave - stomps on anything that may have
// been there before. It is up to the OS to keep order
void _waveplay(WaveT begin, WaveT end);
UINT32 _wavedone();
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////




/* lowlevel display - functions */
typedef UCHAR *DisplayT;

void _displaynow(DisplayT display);
void _displayclear(DisplayT display);
/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////




///////////////////////////////////////////////////
/* A/D Converter - functions and data structures */
typedef struct
{
    UINT16 AdcA;
    UINT16 AdcB;
    UINT16 AdcC;
    UINT16 AdcD;
} AdcDataT;


// initialize the adc
void _initadc(void);

// load the data from the adc into the data pointer
void _adcload(AdcDataT *data);

// stop the adc and disable - power save
void _stopadc(void);

////////////////////////////////////////
/* example use
    AdcDataT    adcdata;

    _initadc();

    ...
    _adcload(&adcdata);
    ...

 */

///////////////////////////////////////////////////
///////////////////////////////////////////////////



/* Graphics */
#define COLOR_RAW_0 0
#define COLOR_RAW_1 1
#define COLOR_RAW_2 2
#define COLOR_RAW_3 3

typedef struct
{
    DisplayT    display;
    UCHAR       x;
    UCHAR       y;
    UCHAR       color;
} GraphicsT;

// the contents of the structure are not changed
void _pixelset(const GraphicsT *context);



void cyConsole(char *, long);


//////////////////////////////////   T I M E R   I N T E R F A C E   /////////
UINT32 _timerticks(void);
void _timerset(UINT32 set);

// This is observed from the boot code - changing this will not
// change the ticks.
#define TICKS_PER_SECOND    50


//////////////////////////////////   K E Y P A D   I N T E R F A C E   ///////
// Key press detection
// returns non-zero if any key is pressed
UINT16  _keypressed(void);

// returns non-zero if esc key is pressed
UINT16  _esckey(void);

// returns non-zero if requested key is pressed
// Do not use ASCII here - use the raw key value from defines below
UINT16  _keycheck(UINT16 rawkey);
// example: if (_keycheck(RAW_KEY_SHIFT)) ...


/* Keypad reading - raw */
// returns zero if no key pressed or first key found pressed
// This function reads the keypad raw - and ignores modifiers
UINT16  _rawkey(void);

#define RAW_KEY_NONE    0

#define RAW_KEY_A   6
#define RAW_KEY_B   47
#define RAW_KEY_C   31
#define RAW_KEY_D   22
#define RAW_KEY_E   21
#define RAW_KEY_F   30
#define RAW_KEY_G   38
#define RAW_KEY_H   46
#define RAW_KEY_I   61
#define RAW_KEY_J   54
#define RAW_KEY_K   62
#define RAW_KEY_L   70
#define RAW_KEY_M   63
#define RAW_KEY_N   55
#define RAW_KEY_O   69
#define RAW_KEY_P   68
#define RAW_KEY_Q   5
#define RAW_KEY_R   29
#define RAW_KEY_S   14
#define RAW_KEY_T   37
#define RAW_KEY_U   53
#define RAW_KEY_V   39
#define RAW_KEY_W   13
#define RAW_KEY_X   23
#define RAW_KEY_Y   45
#define RAW_KEY_Z   15

#define RAW_KEY_0   59
#define RAW_KEY_1   26
#define RAW_KEY_2   12
#define RAW_KEY_3   20
#define RAW_KEY_4   28
#define RAW_KEY_5   36
#define RAW_KEY_6   44
#define RAW_KEY_7   52
#define RAW_KEY_8   60
#define RAW_KEY_9   67

#define RAW_KEY_F1  49
#define RAW_KEY_F2  41
#define RAW_KEY_F3  18
#define RAW_KEY_F4  25
#define RAW_KEY_F5  17
#define RAW_KEY_F6  9
#define RAW_KEY_F7  1

// raw arrow keys
#define RAW_KEY_AU  10  // Arrow Up
#define RAW_KEY_AD  34  // Arrow Down
#define RAW_KEY_AL  4   // Arrow Left
#define RAW_KEY_AR  33  // Arror Right


#define RAW_KEY_FN      16  // Function - modifier
#define RAW_KEY_SHIFT   8   // Shift - modifier


#define RAW_KEY_DEL     3   // delete
#define RAW_KEY_TAB     27  // tab
#define RAW_KEY_INS     11  // insert
#define RAW_KEY_BKSP    51  // backspace
#define RAW_KEY_HELP    24  // question mark
#define RAW_KEY_ENTER   43  // enter
#define RAW_KEY_SPACE   19  // space
#define RAW_KEY_SELECT  35  // select


#define RAW_KEY_LBKT    32  // left bracket     [
#define RAW_KEY_RBKT    40  // right bracket    ]
#define RAW_KEY_BSLASH  48  // back slash
#define RAW_KEY_FSLASH  50  // forward slash    /
#define RAW_KEY_SEMI    42  // semicolon        ;
#define RAW_KEY_EQUAL   66  // equals sign      =
#define RAW_KEY_COMMA   71  // comma            ,
#define RAW_KEY_QUOTE   65  // single quote     '
#define RAW_KEY_DASH    57  // dash             -
#define RAW_KEY_PERIOD  58  // period           .



/////  A S C I I - K E Y

// translates raw key into ascii equiv
extern const UCHAR raw2ascii[];         // without the shift key
extern const UCHAR raw2asciiSHIFTED[];  // use when shift key is pressed
// use like: key = _rawkey(); key = raw2ascii[key];


// Use this to interpret the keypad into ASCII values

typedef UCHAR KeyT;


// The keys are read by addressing an area of memory
KeyT asciikey(void);



/////  J O Y S T I C K  
// 
// The Joystick is different because it allows more than one key at a time
// for a joystick that means you can read left + up at the same time. The 
// asciikey() or _rawkey() only return one key at a time.

typedef UINT8 JoystickT;


#define KEY_JOY_LEFT    0x01
#define KEY_JOY_UP      0x02
#define KEY_JOY_DOWN    0x04
#define KEY_JOY_RIGHT   0x08

JoystickT joystick(void);
// use: JoystickT joy = joystick(); 




/////////////////////////////////////////////////


// realtime clock interface

typedef struct
{
    UINT8 hundredths;   // BCD format
    UINT8 seconds;      // BCD format
    UINT8 minutes;      // BCD format
    UINT8 hours;        // BCD format
} RtcT, *pRtcT;


// not thread safe or reentrant
// OS should guard this from multiple entry
void _rtcinit(void);
void _rtcget(pRtcT buffer);


//////////////////////////////////   L E D   I N T E R F A C E   /////////
void _ledgreenon(void);     // turn on the green LED
void _ledgreenoff(void);    // turn off the green LED
void _ledredon(void);       // turn on the red LED
void _ledredoff(void);      // turn off the red LED



#endif
