#ifndef __RS232_H_
#define __RS232_H_




#define RS232_FRAMING_ERROR 0x01
#define RS232_PARITY_ERROR  0x02
#define RS232_OVERRUN_ERROR 0x04


UINT8 SerialError(void);
// The return value can have flags set if errors are present
// error = SerialError(); if (error & RS232_PARITY_ERROR) service it
// The errors are reset in this function



// Use these two functions to start and stop using the serial port
void SerialBegin(void);
void SerialEnd(void);


StatusT SerialRead(unsigned char *byte);
// read one byte - ST_OK if data, ST_EMPTY if queue is empty


StatusT SerialSend(UCHAR data);
// Send one byte out the serial port, check return for data buffer full


StatusT SerialClose(void);
// All done will the serial port - all resources will be discarded


void SerialBreak(void);
// Send break

////////////////////////////////////////////////////////////////////////////
// Could add more formats, etc
typedef enum
{
    RS232_FORMAT_8N1 = 1,   // 8 bits - no parity   - 1 stop bit
    RS232_FORMAT_8E1,       // 8 bits - even parity - 1 stop bit
    RS232_FORMAT_8D1,       // 8 bits - odd parity  - 1 stop bit
} FormatT;


typedef enum
{
    RS232_BAUD_2400 = 1,    // this is the lowest without code changes
    RS232_BAUD_4800,
    RS232_BAUD_9600,
    RS232_BAUD_19200,       // note: matrixorbital LCDs use this rate
    RS232_BAUD_38400,
    RS232_BAUD_57600,       // note: cybiko classic default rate
    RS232_BAUD_115200,      // this is fastest usable baud rate
} BaudT;


// Not being used now - the linker has a problem linking with relaxed on
// Handshaking is Softare only using XON (ASCII ctrl Q) and XOFF (^S)
typedef enum
{
    HANDSHAKE_OFF,      // Fastest - uses stripped down ISR 
    HANDSHAKE_ON        // Automatically sends XON and XOFF when needed
} HandshakeT;



StatusT SerialOpen(BaudT baud, FormatT format, UINT32 size);
// Open serial port for use, claim ownership
// These selections persist through SerialBegin and SerialEnd.
// To make changes, the port must be closed and reopened
// Parameters:
// baud - baudrate; chose from enumeration
// format - see enumeration
// hs - handshake
// size - the size for BOTH rx and tx queues



#endif

