#ifndef _SURFACE_H_
#define _SURFACE_H_


typedef enum
{
    SURFACE_SCROLL_UP,
    SURFACE_SCROLL_DOWN
} SurfaceScrollT;


typedef struct
{
    unsigned char fgPattern;    // foreground pattern to set to
    unsigned char bgPattern;    // background pattern to clear to
    unsigned cols;              // columns are 8bits = 4 pixels, byte aligned
    unsigned rows;
    unsigned char *buffer;
} SurfaceT, *pSurfaceT;


typedef enum
{
    DRAW_MODE_CLEAR     = 0x0000,
    DRAW_MODE_OVER      = 0x0001,
    DRAW_MODE_FILLED    = 0x0002,
//    DRAW_MODE_OVER      = 0x0004,
} ModeT;


// Set the background color/pattern
void SurfaceBackground(SurfaceT *surface, unsigned char pattern);

// Set the foreground color/pattern
void SurfaceForeground(SurfaceT *surface, unsigned char pattern);

// Clear the entire surface to the background pattern
void SurfaceClear(SurfaceT *surface);

// scroll the surface
void SurfaceScroll(SurfaceT *surface, unsigned amount, SurfaceScrollT scroll);

// Create a surface from memory operation
pSurfaceT SurfaceCreate(unsigned cols, unsigned rows);

// Destroy a surface made with SectionCreate
void SurfaceDestroy(SurfaceT *surface);

// Move the surface to the Display location of the same size
void SurfaceDisplay(SurfaceT const *surface, unsigned col, unsigned row);

// Blit one surface to another
void SurfaceBlit(SurfaceT *src, SurfaceT *dst, unsigned col, unsigned row);

// Set an area of the surface to pattern
void SurfaceRect(SurfaceT *surface,
    unsigned col,
    unsigned row,
    unsigned width,
    unsigned height,
    unsigned char pattern);




#endif



