/***********************************************************************/
/*  This file is part of the C166 Compiler package                     */
/*  Copyright KEIL ELEKTRONIK GmbH 1993 - 1999                         */
/***********************************************************************/
/*                                                                     */
/*  FREE.C:                                                            */
/*                                                                     */
/*  To translate this file use C166 with the following invocation:     */
/*                                                                     */
/*     C166 FREE.C  <memory model>                                     */
/*                                                                     */
/*  To link the modified FREE.OBJ file to your application use the     */
/*  following L166 invocation:                                         */
/*                                                                     */
/*     L166 <your object file list>, FREE.OBJ <controls>               */
/*                                                                     */
/***********************************************************************/

#include <stdlib.h>
#include "malloc.h"


extern void * __mp__;       /* Memory Pool Header */


void free (void  *memp)  {
/*  FREE attempts to organize Q, P0, and P so that Q < P0 < P.  Then, P0 is
 *  inserted into the free list so that the list is maintained in address
 *  order.
 *
 *  FREE also attempts to consolidate small blocks into the largest block
 *  possible.  So, after allocating all memory and freeing all memory,
 *  you will have a single block that is the size of the memory pool.  The
 *  overhead for the merge is very minimal.                                */

    struct __mp__ *q;                  /* ptr to free block */
    struct __mp__ *p;                  /* q->next */

#define p0 ((struct __mp__ *) memp)  /* block to free */

/*  If the user tried to free NULL, get out now.  Otherwise, get the address
 *  of the header of the memp block (P0).  Then, try to locate Q and P such
 *  that Q < P0 < P.                                                       */

    if (!memp)  return;

    //p0 = &p0 [-1];                /* get address of header */
    memp = &p0 [-1];                /* get address of header */

/*  Initialize.  Q = Location of first available block.                    */
    q = __mp__;
    if (!q)
    {
        __mp__ = p0;
        return;
    }

/*  B2. Advance P. Hop through the list until we find a free block that is
 *  located in memory AFTER the block we're trying to free.                */
    for (;;)
    {
        p = q->next;
        if (!p || ((void *)p > memp)) break;
        q = p;
    }

/*  B3. Check upper bound. If P0 and P are contiguous, merge block P into
 *  block P0.                                                              */

    if (p && ((((char *)p0) + p0->len + HLEN) == (char *) p))
    {
        p0->len += p->len + HLEN;
        p0->next = p->next;
    }
    else
    {
        p0->next = p;
    }


/*  B4. Check lower bound. If Q and P0 are contiguous, merge P0 into Q.  */

    if ((((char *)q) + q->len + HLEN) == (char *) p0)
    {
        q->len += p0->len + HLEN;
        q->next = p0->next;
    }
    else
    {
        q->next = p0;
    }
}

