/***********************************************************************/
/*  This file is part of the C166 Compiler package                     */
/*  Copyright KEIL ELEKTRONIK GmbH 1993 - 1999                         */
/***********************************************************************/
/*                                                                     */
/*  INIT_MEM.C:                                                        */
/*                                                                     */
/*  To translate this file use C166 with the following invocation:     */
/*                                                                     */
/*     C166 INIT_MEM.C <memory model>                                  */
/*                                                                     */
/*  To link the modified INIT_MEM.OBJ file to your application use the */
/*  following L166 invocation:                                         */
/*                                                                     */
/*     L166 <your object file list>, INIT_MEM.OBJ <controls>           */
/*                                                                     */
/***********************************************************************/

#include <stdlib.h>
#include "malloc.h"


// This is the storage for where the memory pool is
void * __mp__;    /* Memory Pool Head */


/*
 *  Memory pool headers.  __mp__ points to the first available block or is
 *  NULL if there are no free blocks.
 *
 *  Note that the list is maintained in address order.  __mp__ points to the
 *  block with the lowest address.  That block points to the block with the
 *  next higher address and so on.
 */

void init_mempool (
    void  *pool,    /* address of the memory pool  */
    size_t size)      /* size of the pool in bytes   */
{

/*  Set the __mp__ to point to the beginning of the pool and set
 *  the pool size.                                                         */

    __mp__ = pool;

/*  Set the link of the block in the pool to NULL (since it's the only
 *  block) and initialize the size of its data area.                       */

    ((struct __mp__ *) pool)->next       = NULL;
    ((struct __mp__ *) pool)->len        = size - HLEN;
}


