;; libgcc routines for the Hitachi H8/300 CPU.
;; Contributed by Steve Chamberlain <sac@cygnus.com>
;; Optimizations by Toshiyasu Morita <toshiyasu.morita@hsa.hitachi.com>

/* Copyright (C) 1994, 2000, 2001, 2002 Free Software Foundation, Inc.

This file is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

In addition to the permissions in the GNU General Public License, the
Free Software Foundation gives you unlimited permission to link the
compiled version of this file into combinations with other programs,
and to distribute those combinations without any restriction coming
from the use of this file.  (The General Public License restrictions
do apply in other respects; for example, they cover modification of
the file, and distribution when not linked into a combine
executable.)

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

/* Assembler register definitions.  */

#define A0 r0
#define A0L r0l
#define A0H r0h

#define A1 r1
#define A1L r1l
#define A1H r1h

#define A2 r2
#define A2L r2l
#define A2H r2h

#define A3 r3
#define A3L r3l
#define A3H r3h

#define S0 r4
#define S0L r4l
#define S0H r4h

#define S1 r5
#define S1L r5l
#define S1H r5h

#define S2 r6
#define S2L r6l
#define S2H r6h


#define MOVP	mov.l	/* pointers are 32 bits */
#define ADDP	add.l
#define CMPP	cmp.l
#define PUSHP	push.l
#define POPP	pop.l

#define A0P	er0
#define A1P	er1
#define A2P	er2
#define A3P	er3
#define S0P	er4
#define S1P	er5
#define S2P	er6

#define A0E	e0
#define A1E	e1
#define A2E	e2
#define A3E	e3


    .h8300s


divnorm:
    mov.b	#0,S2L		; keep the sign in S2
    mov.l	A0P,A0P		; is the numerator -ve
    bge	postive

    neg.l	A0P		; negate arg
    mov.b	#1,S2L		; the sign will be -ve

postive:
    mov.l	A1P,A1P		; is the denominator -ve
    bge	postive2

    neg.l	A1P		; negate arg
    xor.b	#1,S2L		; toggle result sign

postive2:
    rts


;; Basically the same, except that the sign of the divisor determines
;; the sign.
modnorm:
    mov.b	#0,S2L		; keep the sign in S2
    mov.l	A0P,A0P		; is the numerator -ve
    bge	mpostive

    neg.l	A0P		; negate arg
    mov.b	#1,S2L		; the sign will be -ve

mpostive:
    mov.l	A1P,A1P		; is the denominator -ve
    bge	mpostive2

    neg.l	A1P		; negate arg

mpostive2:
    rts



; numerator in A0/A1
; denominator in A2/A3
    .global ___modsi3
___modsi3:
    PUSHP	S2P
    PUSHP	S0P
    PUSHP	S1P

    bsr	modnorm
    bsr	divmodsi4
    mov.l	S0P,A0P
    bra	exitdiv

    .global	___udivsi3
___udivsi3:
    PUSHP	S2P
    PUSHP	S0P
    PUSHP	S1P
    mov.b	#0,S2L	; keep sign low
    bsr	divmodsi4
    bra	exitdiv

    .global	___umodsi3
___umodsi3:
    PUSHP	S2P
    PUSHP	S0P
    PUSHP	S1P
    mov.b	#0,S2L	; keep sign low
    bsr	divmodsi4
    mov.l	S0P,A0P
    bra	exitdiv

    .global	___divsi3
___divsi3:
    PUSHP	S2P
    PUSHP	S0P
    PUSHP	S1P
    jsr	divnorm
    jsr	divmodsi4

    ; examine what the sign should be
exitdiv:
	POPP	S1P
	POPP	S0P

	or	S2L,S2L
	beq	reti

	; should be -ve
	neg.l	A0P

reti:
	POPP	S2P
	rts

	; takes A0/A1 numerator (A0P for H8/300H)
	; A2/A3 denominator (A1P for H8/300H)
	; returns A0/A1 quotient (A0P for H8/300H)
	; S0/S1 remainder (S0P for H8/300H)
	; trashes S2

divmodsi4:
	sub.l	S0P,S0P		; zero play area
	mov.w	A1E,A1E		; denominator top word 0?
	bne	DenHighZero

	; do it the easy way, see page 107 in manual
	mov.w	A0E,A2
	extu.l	A2P
	divxu.w	A1,A2P
	mov.w	A2E,A0E
	divxu.w	A1,A0P
	mov.w	A0E,S0
	mov.w	A2,A0E
	extu.l	S0P
	rts

DenHighZero:
	mov.w	A0E,A2
	mov.b	A2H,S0L
	mov.b	A2L,A2H
	mov.b	A0H,A2L
	mov.w	A2,A0E
	mov.b	A0L,A0H
	mov.b	#0,A0L
	mov.b	#24,S2H		; only do 24 iterations

nextbit:
	shll.l	A0P		; double the answer guess
	rotxl.l	S0P		; double remainder
	sub.l	A1P,S0P		; does it all fit?
	bhs	setone

	add.l	A1P,S0P		; no, restore mistake
	dec	S2H
	bne	nextbit
	rts

setone:
	inc	A0L
	dec	S2H
	bne	nextbit
	rts




	.section .text
	.align 2


;
; mulsi3 for H8/300H - based on Hitachi SH implementation
;
; by Toshiyasu Morita
;
; Old code:
;
; 16b * 16b = 372 states (worst case)
; 32b * 32b = 724 states (worst case)
;
; New code:
;
; 16b * 16b =  48 states
; 16b * 32b =  72 states
; 32b * 32b =  92 states
;

    .global	___mulsi3
___mulsi3:
    mov.w	r1,r2   ; ( 2 states) b * d
    mulxu	r0,er2  ; (22 states)

    mov.w	e0,r3   ; ( 2 states) a * d
    beq	L_skip1 ; ( 4 states)
    mulxu	r1,er3  ; (22 states)
    add.w	r3,e2   ; ( 2 states)

L_skip1:
    mov.w	e1,r3   ; ( 2 states) c * b
    beq	L_skip2 ; ( 4 states)
    mulxu	r0,er3  ; (22 states)
    add.w	r3,e2   ; ( 2 states)

L_skip2:
    mov.l	er2,er0	; ( 2 states)
    rts		; (10 states)



