/***********************************************************************/
/*  This file is part of the C166 Compiler package                     */
/*  Copyright KEIL ELEKTRONIK GmbH 1993 - 1999                         */
/***********************************************************************/
/***********************************************************************/

#include <stdlib.h>
#include "malloc.h"     // local malloc definitions


extern void * __mp__;       /* Memory Pool Header */


void *malloc (size_t size)
{
    struct __mp__ *q;  /* ptr to free block */
    struct __mp__ *p;  /* q->next */
    size_t k;            /* space remaining in the allocated block */

/*  Initialization:  Q is the pointer to the next available block.          */
    q = (struct __mp__ *) &__mp__;

/*  End-Of-List:  P points to the next block.  If that block DNE (P==NULL),
 *  we are at the end of the list.                                          */
    for (;;)
    {
        p = q->next;
        if (!p)  return (NULL); /* FAILURE */

/*  Found Space:  If block is large enough, reserve it.  Otherwise, copy P
 *  to Q and try the next free block.                                       */
        if (p->len >= size) break;
        q = p;
    }

/*  Reserve P:  Use at least part of the P block to satisfy the allocation
 *  request.  At this time, the following pointers are setup:
 *  P points to the block from which we allocate Q->next points to P        */

    k = p->len - size;		/* calc. remaining bytes in block */

    /* rem. bytes too small for new block */
    if (k < MALLOC_MIN_BLOCK)
    {
        q->next = p->next;
        return (&p[1]);     /* SUCCESS */
    }

/*  Split P Block:  If P is larger than we need, we split P into two blocks:
 *  the leftover space and the allocated space.  That means, we need to
 *  create a header in the allocated space.                                 */

    k -= HLEN;
    p->len = k;

    q = (struct __mp__ *) (((char *) (&p[1])) + k);
    q->len = size;

    return (&q[1]);           /* SUCCESS */
}


