

// need memory for memmove, memcpy, memset, etc...
#include "string.h"



// the counts can be any number
// the size MUST have the last 3 bits clear
// the size MUST strictly increase
#define BIN0_SIZE   (1 << 4)
#define BIN0_COUNT  1000

#define BIN1_SIZE   (1 << 5)
#define BIN1_COUNT  1000

// how about room for fonts or bitmaps or midi files
#define BIN2_SIZE   (1 << 9)
#define BIN2_COUNT  200

// size
#define BIN3_SIZE   (1 << 11)
#define BIN3_COUNT  20

// since the cybiko has a display of 4000 bytes + plus stuff
#define BIN4_SIZE   (1 << 12)
#define BIN4_COUNT  5


// 'used' is how many bytes used in this bin
typedef unsigned int UsedT;


// This block holds everything
#define MAKE_BLOCK(NUMBER, COUNT, SIZE) \
static struct                           \
{                                       \
    unsigned    used;                   \
    unsigned    next;                   \
    void        *nextptr;               \
    struct                              \
    {                                   \
        UsedT       used;               \
        char        space[SIZE];        \
    } bin[COUNT];                       \
} Block##NUMBER;                        \
                                        \
                                        \
// Search bins - call here infers that size was small enough    \
void *GetBin##NUMBER(unsigned long size)                             \
{                                                               \
    void *bin;                                                  \
                                                                \
    // anything left?                                           \
    if (Block##NUMBER.used <= COUNT)                            \
    {                                                           \
        // assign the open bin - round the size                 \
        size = NUMBER | ((size + 0x07) & ~(0x07));              \
                                                                \
        // grab the next open                                   \
        bin = (void *)((char *)Block##NUMBER.nextptr + sizeof(UsedT));\
        (UsedT *)Block##NUMBER.nextptr = size;                  \
                                                                \
        // decrement the usage count - still have some left?    \
        if (++Block##NUMBER.used < COUNT)                         \
        {                                                       \
            // since some left, there MUST be an open block     \
            while (Block##NUMBER.bin[Block##NUMBER.next].used)  \
            {                                                   \
                Block##NUMBER.next++;                           \
                if (Block##NUMBER.next >= COUNT)                \
                {                                               \
                    Block##NUMBER.next = 0;                     \
                }                                               \
            }                                                   \
                                                                \
            // now make a next pointer out of the next          \
            Block##NUMBER.nextptr = (void *)&Block##NUMBER.bin[Block##NUMBER.next];\
        }                                                       \
    }                                                           \
    else                                                        \
    {                                                           \
        return (void *)0;                                       \
    }                                                           \
}                                                               \



// ONLY 8 blocks max!!
MAKE_BLOCK(0, BIN0_COUNT, BIN0_SIZE);
MAKE_BLOCK(1, BIN1_COUNT, BIN1_SIZE);
MAKE_BLOCK(2, BIN2_COUNT, BIN2_SIZE);
MAKE_BLOCK(3, BIN3_COUNT, BIN3_SIZE);
MAKE_BLOCK(4, BIN4_COUNT, BIN4_SIZE);
//MAKE_BLOCK(5, BIN5_COUNT, BIN5_SIZE);
//MAKE_BLOCK(6, BIN6_COUNT, BIN6_SIZE);
//MAKE_BLOCK(7, BIN7_COUNT, BIN7_SIZE);


void *(blocks[]) = { &Block0, &Block1, &Block2, &Block3, &Block4 };





// design note: bins must be in order from smaller to larger
void *malloc(unsigned long size)
{
    void *bin;

    if (size <= BIN0_SIZE)
    {

        bin = GetBin0(size);
        if (bin)
        {
            return bin;
        }
    }

    if (size <= BIN1_SIZE)
    {
        bin = GetBin1(size);
        if (bin)
        {
            return bin;
        }
    }

    if (size <= BIN2_SIZE)
    {
        bin = GetBin2(size);
        if (bin)
        {
            return bin;
        }
    }

    if (size <= BIN3_SIZE)
    {
        bin = GetBin3(size);
        if (bin)
        {
            return bin;
        }
    }

    if (size <= BIN4_SIZE)
    {
        bin = GetBin4(size);
        if (bin)
        {
            return bin;
        }
    }

    // failed to get memory
    return (void *)0;

}




// null pointers do nothing
// DONT FREE MORE THAN ONCE!!!!
// use the data in the pointer to set the next
void free(void *pointer)
{
    if (pointer)
    {
        unsigned block;

        // the block number is inside the size of the bin
        block = *((UsedT *)pointer - 1) & 0x07;

        // no longer used - point back 2 chars (pointer math!) and zero it
        *((UsedT *)pointer - 1) = 0;

        // assign the 'next' pointer to the newly freed pointer
        //blocks[block].next = pointer;
        (*((unsigned *)blocks[block]+2))--;  // less one used

        // set the next pointer
        (*(void**)blocks[block]) = ((UsedT *)pointer - 1);

    }
}



/*
calloc() allocates memory for an array of nmemb elements of size bytes each and
returns a pointer to the allocated memory. The memory is set to zero.
*/
void *calloc(unsigned long nmemb, unsigned long size)
{
    void *cleared;

    cleared = malloc(nmemb * size);
    if (cleared)
    {
        memset(cleared, 0, nmemb * size);
    }

    return cleared;

}




/*
realloc() changes the size of the memory block pointed to by ptr to size bytes.
The contents will be unchanged to the minimum of the old and new sizes; newly
allocated memory will be uninitialized. If ptr is NULL, the call is equivalent
to malloc(size); if size is equal to zero, the call is equivalent to free(ptr).
Unless ptr is NULL, it must have been returned by an earlier call to malloc(),
calloc() or realloc().
*/

void *realloc(void *older, unsigned NewerSize)
{
    void *newer;

    if (!older)
    {
        // act just like malloc if older pointer is NULL
        return malloc(NewerSize);
    }

    if (!NewerSize)
    {
        // act just like free if size is zero
        free(older);
        return (void *)0;
    }

    // resize and copy smallest area
    newer = malloc(NewerSize);
    if (newer)
    {
        UsedT OlderSize;

        OlderSize = *((UsedT *)older-1) & ~(0x07);
        memmove(newer, older, min(OlderSize, NewerSize));

        free(older);
    }

    return newer;
}





