;
;    Cyborn OS - Copyright (C) 2004 Steve Gotthardt - Gotthardt Designs
;
;    This file is part of the Cyborn distribution.
;
;    Cyborn is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    Cyborn is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with Cyborn; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    A special exception to the GPL can be applied should you wish to distribute
;    a combined work that includes Cyborn, without being obliged to provide
;    the source code for any proprietary components.
;


; Analog to Digital Converter - ADC

    .h8300s
    .text

    .global __initadc				; export function
    .global __adcload

    .set H8S_ADC_ADCSR, 0x98        ;ADC Control and Status Register
    .set H8S_ADC_ADCR,  0x99        ;ADC Control Register

    .set H8S_ADC_ADDRA, 0xFF90        ;ADC Results for AN0 - 16 bits - 2 bytes
    .set H8S_ADC_ADDRB, 0xFF92        ;ADC Results for AN1 - 16 bits - 2 bytes
    .set H8S_ADC_ADDRC, 0xFF94        ;ADC Results for AN2 - 16 bits - 2 bytes
    .set H8S_ADC_ADDRD, 0xFF96        ;ADC Results for AN3 - 16 bits - 2 bytes

    .set H8S_ADC_CONVS, 0xFFFF90    ;ADC Results for AN0 - 16 bits - 2 bytes



__initadc:  ; no parameters
    ;get the module stops - and unstop the adc
    ;cant talk to other registers in the adc until unstopped
    mov     @0xff3c:16, r0
    and     #0xfd,      r0h
    mov     r0,         @0xff3c:16

    ;set defaults to: scan all 4 channels - initiate scan - continuous mode
    ; first make sure the scan is stopped before setting scan mode (continuous or single)
    bclr    #5,      	@H8S_ADC_ADCSR:8

	; disable external triggering
    xor		r0h, 		r0h					; clear reg
    mov     r0h,        @H8S_ADC_ADCR:8		; control register

	; now set the mode to be continuous and start the conversions
    mov     #0x37,      r0h
    mov     r0h,        @H8S_ADC_ADCSR:8

    rts



__stopadc:  ; no parameters
    ;stop the conversions - dont really have to do this
    bclr    #5,      	@H8S_ADC_ADCSR:8

    ; save the interrupt state, disable interrupts before doing module stops
    stc		ccr, 	r1l
    orc		#0x80,	ccr

    ;get the module stops - and stop the adc
    mov     @0xff3c:16, r0
    or      #0x02,      r0h
    mov     r0,         @0xff3c:16

	ldc		r1l,		ccr

    rts



;This whole thing can be reworked to be interrupt driven, etc
;The data can be stored at end of conversion in an ISR context
;Or a function can be registered

;er0 has pointer to data - 16 bits each
__adcload:

    ;spin on conversion
waitadc:
    btst    #7, @H8S_ADC_ADCSR:8    ;bit seven set means conversion complete
    beq     waitadc                 ;clear bit means not done yet

    mov     #H8S_ADC_CONVS, er1

;note that upper MUST be read before lower byte - cyos does it backwards!

    mov     @er1+,          r2
    mov     r2,             @er0
    adds    #2,             er0

    mov     @er1+,          r2
    mov     r2,             @er0
    adds    #2,             er0

    mov     @er1+,          r2
    mov     r2,             @er0
    adds    #2,             er0

    mov     @er1,           r2
    mov     r2,             @er0


    ;clear conversion finished bit
    bclr    #7, @H8S_ADC_ADCSR:8

    ;start/continue conversion
    bset    #5, @H8S_ADC_ADCSR:8

    rts



