

// Internal Data flash driver
#define DF_CS0_ASSERT   asm(bclr 0x123456, X)
#define DF_CS0_DEASSERT asm(bset 0x123456, X)


// For external Data Flash
#define DF_CS1_ASSERT   asm(bclr 0x123456, X)
#define DF_CS1_DEASSERT asm(bset 0x123456, X)

#define DF_CS2_ASSERT   asm(bclr 0x123456, X)
#define DF_CS2_DEASSERT asm(bset 0x123456, X)


StatusT
IDATAFLASH_init(DeviceT *device)
{
    StatusT         status;
    DeviceGeometryT geometry;

    // need to fill in the device for internal dataflash
    // This should be an ATMEL Data flash

    DF_CS0_ASSERT;
    status = ATMEL_detect(&geometry);
    DF_CS0_DEASSERT;

    if  (status == ST_OK)
    {
        // found an atmel chip and have geometry for it
        // Geometry
        device->pagesize = geometry.pagesize;
        device->pages = geometry.pages;

        // Functions
        device->read = IDF_read;
        device->readpage = IDF_readpage;
        device->write = IDF_write;
        device->writepage = IDF_writepage;
    }

    return status;
}

// this is a wrapper for the ATMEL functions with simple chip select
static StatusT
IDF_read(PageT page, size_t start, size_t size, char *buffer)
{
    DF_CS0_ASSERT;

    // Pass the parameters through - hope the compiler is smart
    ATMEL_read(PageT page, size_t start, size_t size, char *buffer);

    DF_CS0_DEASSERT;
}



// this is a wrapper for the ATMEL functions with simple chip select
static StatusT
IDF_readpage(PageT page, char *buffer)
{
    DF_CS0_ASSERT;

    // Pass the parameters through - hope the compiler is smart
    ATMEL_readpage(PageT page, char *buffer);

    DF_CS0_DEASSERT;
}



// External Data Flash
// this is a wrapper for the ATMEL functions with simple chip select
static StatusT
IDF_read(PageT page, size_t offset, size_t size, char *buffer)
{
    DF_CS0_ASSERT;

    // Pass the parameters through - hope the compiler is smart
    ATMEL_read(PageT page, char *buffer);

    DF_CS0_DEASSERT;
}




// example ATMEL read



