;
;    Cyborn OS - Copyright (C) 2004 Steve Gotthardt - Gotthardt Designs
;
;    This file is part of the Cyborn distribution.
;
;    Cyborn is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    Cyborn is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with Cyborn; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    A special exception to the GPL can be applied should you wish to distribute
;    a combined work that includes Cyborn, without being obliged to provide
;    the source code for any proprietary components.  
;

;   Since the registers depend upon time and have a 'state': a register is set
;   to point to others, _displaynow is not re-entrant. The OS should guard this
;   function.

    .h8300s
    .text

    .global __displaynow
    .global __displayclear

    .set HD66421_BASE_SET,  0x600000
    .set HD66421_BASE_USE,  0x600001
    .set HD66421_REG_R0,    0x00  	;Control Register 1
    .set HD66421_REG_R1,    0x01	;Contorl Register 2
    .set HD66421_REG_R2,    0x02	;X address Register
    .set HD66421_REG_R3,    0x03	;Y address Register
    .set HD66421_REG_R4,    0x04	;Display RAM access register
    .set HD66421_REG_R5,    0x05	;Display start line register
    .set HD66421_REG_R6,    0x06	;Blink start line register
    .set HD66421_REG_R7,    0x07	;Blink end line register
    .set HD66421_REG_R8,    0x08	;Blink register 1
    .set HD66421_REG_R9,    0x09	;Blink register 2
    .set HD66421_REG_R10,   0x0A	;Blink register 3
    .set HD66421_REG_R11,   0x0B	;Partial display block register
    .set HD66421_REG_R12,   0x0C	;Gray scale pallete 1 (0,0)
    .set HD66421_REG_R13,   0x0D	;Gray scale pallete 2 (0,1)
    .set HD66421_REG_R14,   0x0E	;Gray scale pallete 3 (1,0)
    .set HD66421_REG_R15,   0x0F	;Gray scale pallete 4 (1,1)
    .set HD66421_REG_R16,   0x10	;Contrast control register
    .set HD66421_REG_R17,   0x11	;Plane selection register



;low level display mover
;TODO: change this to use DTC

; er0 should contain display buffer pointer

__displaynow:
    ;set the X and Y start to 0
    mov     #HD66421_REG_R2, r1l        ; point to x register
    mov     r1l, @HD66421_BASE_SET:32
    mov     #0, r1l
    mov     r1l, @HD66421_BASE_USE:32   ; set x register to zero


    ;do 100 rows/lines for x (40 bytes)
    mov     #99,   r2l
    
nextrow:
    ;set the Y register to row
    mov     #HD66421_REG_R3, r1l
    mov     r1l, @HD66421_BASE_SET:32
    mov     r2l, @HD66421_BASE_USE:32

    ;index to the RAM write access register
    mov     #HD66421_REG_R4, r1l
    mov     r1l, @HD66421_BASE_SET:32

    mov     #HD66421_BASE_USE, er1  ;for speed

    mov     #20, e2    ;transfer all bytes for the row
    
nextbyte:
    mov     @er0+,  r2h
    mov     r2h,    @er1

    mov     @er0+,  r2h
    mov     r2h,    @er1

    dec     #1,     e2
    bne     nextbyte

    dec     r2l
    bpl     nextrow

    rts



    
;er0 is display pointer
__displayclear:
    add     #4000,  er0     ;4000 bytes to clear
    mov     #250,   r1l     ;250 * 16 -> 4000
    sub     er2,    er2
cleardis:
    mov     er2,    @-er0
    mov     er2,    @-er0
    mov     er2,    @-er0
    mov     er2,    @-er0
    
    dec     r1l
    bne     cleardis
    rts



