
#include "filesys.h"



#define DRIVES_MAX  5

// Only so many DataSystems can be created
DataSystemT DataSystems[DRIVES_MAX];


//#define POS(structure, member) structure




//
// file system implementation
//

// The filesystem has one top directory and the rest come from it
// All entries are seen the same.

// Format of an entry:
// [NAME][TYPE][SIZE][NEXT][DATA][DATA]
// Each entry takes at least one block
// Types of entries:

#define ENTRY_CLASS_DIR     0x40
#define ENTRY_DIR_TOP       (ENTRY_CLASS_DIR | 0x01)
#define ENTRY_DIR           (ENTRY_CLASS_DIR | 0x02)

#define ENTRY_CLASS_FILE    0x80
#define ENTRY_FILE_DOS      (ENTRY_CLASS_FILE | 0x01)
#define ENTRY_FILE_UNIX     (ENTRY_CLASS_FILE | 0x02)
#define ENTRY_FILE_BIN      (ENTRY_CLASS_FILE | 0x03)



// what block is the top dir in?
#define FILESYS_TOPDIR_PAGE 0

#define DEVICE_ID_INTERNAL 0
#define DEVICE_ID_EXTERNAL 1




// assumes: must have a valid file system on the device
// index is a global...
void
IndexFromPage(PageT page)
{
    EntryHeaderT data;

    device->readfront(page, &data, sizeof(EntryHeaderT));

    if (data.type == ENTRY_TYPE_FILE)
    {
        for (;;)
        {
            index[next] = data.next;

            // no more data to follow -
            if (data == 0) return;

            // follow the data
            next = data;

            // read the data header (files cannot point to dirs)
            device->read(next, data, sizeof(DataHeaderT));
        }
    }
    else    // another directory
    {
        // recurse
        IndexFromPage(data.next);
    }
}



#define FILESYSTEM_FLAG_REMOVEABLE 0x80





// NEEDS TO BE GUARDED
static StatusT
GetFreePage(FileT file, PageT *freepage)
{
    StatusT status;

    status = ST_OK;

// CriticalEnter(file->critical);

    // find an open block on the device
    *freepage = file.location->drive->device.takepage();


// CriticalExit(file->critical);


    return status;
}






static StatusT
DataSystemGet(DriveLetterT letter, DataSystemT **system)
{
    int index;

    for (index = 0; index < drives; index++)
    {
        if (DataSystems[index].letter == letter)
        {
            *system = &DataSystems[index];
            return ST_OK;
        }
    }

    return ST_NOTFOUND;
}



// given an entire path, find the location
// The location contains the drive and physical block
static StatusT
LocationFromPath(PathT *path, EntryLocationT *location)
{
    StatusT status;


    // slow part of this system
    // start at the top dir and parse it - look for the block
    // we have a starting point and a path - use '/' for delimiter
    ASSERT(path != NULL);

    // the next char better be a /
    if (*path++ != '/') return ST_BADPATH;

    // what filesytem and device is associated with the drive name?
    status = DataSystemGet(*path++, &(location->drive));
    if (status != ST_OK) return status;

    // the next char better be a /
    if (*path++ != '/') return ST_BADPATH;

    //  example: top/dir/entry or dir/entry
    //  note: entry can be a file or dir

    // start the location at the top
    // The start is always the top dir entry
    location->page = location->drive.start;
    location->directory = location->drive.start;

    for (;;)
    {
        StatusT         status;
        FilesysNameT    name;

        // strip out a name from the path
        // nasty work using strings in 'C'
        strcpy(name, path);
        path += strlen(name);

        // find the name in the directory - return the page its in
        // An error will return if location->dir is not a dir
        // Page will be updated to entry found in the dir
        status = location->drive.search(location, name);
        if (status != ST_OK) return status;

        // are we done yet?
        if (*path == NULL) break;

        // go to the next directory
        location->directory = location->page;

    }

    // if we got this far, there were no errors and nothing missing
    // block should have the block number of where path pointed to

    return ST_OK;

}



    // check for a duplicate entry
    status = entrysearch(dir, &search);
    if (status == ST_OK) return ST_DUPLICATE;


// TODO:


//
StatusT
EntryBlockAdd(char *data, PageT page)
{
    short size;

    // we have a buffer that needs to have data added to it
    // There is a DataEntryHeaderT
    size = (DataPageT *)data->size;

    // room to put a short in the buffer?
    if (size >= DATA_ENTRY_SIZE - sizeof(PageT)) return ST_NOROOM;

    // size is ok - add the data and increase the size
    datapointer = data + sizeof(DataPageT) + (DataPageT *)data->size;

    *(short *)datapointer = block;

    (DataPageT *)data->size += sizeof(PageT);

}




// Add an entry to a directory - could be a file or a dir...
// Be careful - only add to a directory
// assume: no duplicate in this directory
StatusT
DirectoryEntryAdd(EntryLocationT *dir, PageT entry)
{
    EntryHeaderT    header;
    StatusT         status;


    // Get the directory header - raw
    dir->drive->device.read(dir->position.page, 0, sizeof(header), &header);

    // we have a buffer that needs to have data added to it
    // room to put a short in the buffer?
    if (header.size >= DATA_ENTRY_SIZE - sizeof(PageT)) return ST_NOROOM;

    // size is ok - add the data and increase the size
    datapointer = data + sizeof(DataHeaderT) + (DataHeaderT *)data->size;

    *(PageT *)datapointer = entry;

    header.size += sizeof(PageT);


    // add the block number into the data list on the dir block
    // save the directory block back
    dir->drive->device.writepage(dir->directory, data);
}




// get the size of data in the entry from the entry header
#define ENTRY_SIZE(entry) ((EntryHeaderT *)entry->size)
#define ENTRY_TYPE(entry) ((EntryHeaderT *)entry->type)



// adding an entry
// Error checking:
// - Can only add entry to a directory
// - Directory can only hold a limited number of entries
// - Entry name cannot have same name as another in same directory
// - Filesystem must have free page for new entry



// This is the System API entry - the DataSystem to create it on will be
// determined and called
StatusT
EntryCreate(PathT *path, EntryNameT name, EntryTypeT type)
{
    StatusT         status;
    PageT           block;
    EntryLocationT  location;
    EntryHeaderT    header;
    char            *testpath;
    PageT           *EntryNumber;
    UINT16          DirEntryCount;


    // simple checks
    if (path == NULL)           return ST_NULL;
    if (name == NULL)           return ST_NULL;
    if (type == ENTRY_DIR_TOP)  return ST_ILLEGAL;


    // qualify the path and turn it into a location record
    status = LocationFromPath(path, &location);
    if (status != ST_OK) return status;

    // add the entry via the filesystem type
    status = location->drive->filesystem.entrycreate(&location, name, type);

    //
    return status;
}



typedef struct
{
    UINT8   device;
    UINT8   filesystem;
    UINT16  pages;
    UINT16  start;
    UINT16  flags;
} DSTableEntryT;


typedef struct
{
    char            name[16];
    DSTableEntryT   dstable[8];
} DataSystemInfoT;



// assume: preemption and multitasking off
// stack should be set up - around 500 bytes or so?
// internal flash access needed
// The reality is that there will be only one filesystem: CBFS and one
/// device: the internal data flash.
StatusT
DataSystemInit(void)
{
    StatusT         status;
    DeviceT         *device;
    PageT           *index;
    DataSystemInfoT info;


    // First look in the internal flash using a raw read of the zero page
    // page zero has the 'fstab' - or now what I will call dstab:
    // DataStystem Table and some other markers

    status = FlashRawRead(&info, 0, sizeof(DataSystemInfoT));
    if (status != ST_OK) return status;     // big trouble

    // check the information - is this a Cybiko Reborn?
    if (strncmp(info.name, "CYBORN", 6)) return ST_UNKNOWN;


    // There will always be a DataSystem on the internal flash starting at
    // page one - unless something went wrong.

    // DSTAB starts at 20 and is in this format of 8 bytes:
    // DEVICE   FILESYTEM   PAGES   START   FLAGS
    //   00h        01h     1234h   1234h   ABCDh
    //
    //  device is internal flash
    //  FILESYSTEM is code numbered 1 for CYBORN
    //  pages is number of pages in the filesystem - NOTE pagesize is device
    //  dependant and needs to be recomputed if device is changed
    //  START - what page does the filesystem begin?
    // DEVICE   FILESYTEM   PAGES   START   FLAGS
    //   0          0        0        0     xxxxh   // means invalid entry

    // Internal Flash Device
    // get the device from the list of pointers to the devices
    // Set up the globals to do the transversing via recursion


    // add the data system to the kernel
    for (drives = 0, index = 0; index++; index < DRIVES_MAX)
    {
        if (IS_VALID_DS(info.dstable[index]))
        {
            DataSystems[drives].letter = 'A' + drives;

            DataSystems[drives].filesytem = CBFS;
            DataSystems[drives].pages = info.dstable[index].pages;
            DataSystems[drives].start = info.dstable[index].start;
            DataSystems[drives].device = info.dstable[index].devicetablelookup;


        }
    }

    // drives contains the number of system drives
    // we need to have at least one drive
    if (drives == 0) return ST_NODRIVES;

    // now run any special initializations on the DataSystems
    for (index = 0; index < drives; index++)
    {
        status = DataSystems[index].filesystem->init(&DataSystems[index]);
        if (status != ST_OK) return status;
    }


    // all drives should be ready
    // now run some kind of hotplug task for removable media
    // DataSystemHotplugTask();


    return ST_OK;

}

