
;structure for a wave to play:
;long char *end;
;long char *start;	// note that start is last!!!

;NOTE: These are low level routines with little parameter checking.
;      If you pass in wrong values (ie 4 for a channel number) you WILL
;      mess things up! Higher levels should be checking values and
;      parameters for legal values and checking return values for error codes.


; 4 channel version - doesnt sound so good


       .h8300s
    .text

    ;export these:
    .global __waveplay
    .global __waveinit
    .global __wavedone
    .global _nominal


#define TPU1TGRB        0xFFEA
#define SAMPLE_RATE     8000
#define CLOCK_SPEED     11059200
; Table of time / instructions for processor running at 11MHz
; The interrupt rate needs to be fast enough to sound good and slow enough
;  for other processes to get something done.

;    RATE   Period   States
; 11250Hz     89uS      983
; 11000       91       1005
; 10000      100       1106
;  9000      111       1229
;  8000      125       1382
;; doesn't leave much time for other things if all 4 channels are running...
;; If it takes 90uS to service the interrupt and it happens at 11000 times/Sec
;; then we can be in trouble!!


    .set TCR0,      0xB0    ;Timer 0 Control Register
    .set TCSR0,     0xB2
    .set TCORA0,    0xB4    ;Timer Constant Regsister A0

    .set TSTR,      0xC0    ; for all counters / TPUs
    .set TSYR,      0xC1
    .set MSTPCR,    0xFF3C    ;word

    .set TCR1,      0xE0
    .set TMDR1,     0xE1
    .set TIOR1,     0xE2
    .set TIER1,     0xE4
    .set TSR1,      0xE5
    .set TCNT1,     0xFFE6    ;word
    .set TGR1A,     0xFFE8    ;word
    .set TGR1B,     0xFFEA    ;word


__waveinit:     ;void _waveinit(void);
    ;point the interrupt vector in on-chip RAM to our __waveplay
    ;start the timer that calls our routine at 11KHz
    mov     #__waveISR,    er1
    mov     er1,        @0xEC74:16  ;soft vector for Timer 0 compare A int

;set TCR0 to Compare interrupt A - clock of internal clock / 8 = 1382400
;compare match will clear the timer
    mov     #0x49,      r1l
    mov     r1l,        @TCR0:8

    ; clock / 8 / 8000 = ~ 173 for 8000 Hz sound reproduction
    ;now set the compare register A0 for the desired rate
    mov     #173,       r1l          ;choose this value carefully or get noise
    mov     r1l,        @TCORA0:8


    ;Set up for PWM - since speaker is on TIOCB1 we must use PWM mode 2
    ;turn on the module: TPU
    mov     @MSTPCR:16,     r1
    and     #0xDF,  r1h
    mov     r1,     @MSTPCR:16

    ;make sure counting is off
    bclr    #1,     @TSTR:8

    ;Set up sound control register - TCR1
    ;Output at the point which the counter is cleared to 0 is specified
    ;Clearing source is TGRA compare match because the output is on B
    ;Clock divide by 1 (full speed)
    ;Since output is required on pin TIOCB1, period must be specified in TGRA
    mov     #0x20,  r1l
    mov     r1l,    @TCR1:8

    ;Set the Timer IO Control Register - Control the TGR registers
    ;TGR1B - output compare - should start at 1 and then go to 0 at match
    ;Output of TGR1A is disabled
    mov     #0x50,  r1l
    mov     r1l,    @TIOR1:8

    ; 0x1FF - 9bits
    ; can be changed for other bits - BUT - too
    ; low of a frequency will make noise in output
    mov     #0x167, r1
    mov     r1,     @TGR1A:16

    ; PWM mode 2: bits 0 and 1 set
    mov     #0x03,  r1l
    mov     r1l,    @TMDR1:8

    ;enable counting - PWM runs all the time, waiting for a wave to arrive
    bset    #1,     @TSTR:8

    rts



;fill channel pointer with given value - overwrite without checking
__waveplay:     ;waveplay(WaveT *start, WaveT *finish);
    ;make the channel number an offset
    ;set interrupt mask - ?
    orc    #0x80, ccr


    mov     #channel,  er3
    mov     er1,    @er3        ;move the end to first entry
    adds    #4,     er3
    mov     er0,    @er3        ;move begin to next entry

    ;clear interrupt mask - ?
    andc    #0x7F, ccr

    rts



;on return er0 will contain the number of bytes to go before done
__wavedone:     ;long wavedone(unsigned long channel);
    mov     #channel,       er1
    mov     @er1+,  er0     ;get the end
    mov     @er1,   er1     ;get the start
    sub     er1,    er0
    rts                     ;remaining bytes



    ;interrupt context - all registers have been saved and will be
    ;restored upon return from subroutine - as we were called
    ;FOR SPEED: copy this to on-chip RAM
__waveISR:
    bclr    #6,     @TCSR0:8    ;clear interrupt flag

    mov     #channel,  er0

    mov     @er0+,  er2     ;get the end pointer
    mov     @er0,   er1     ;get the start pointer

    cmp     er1,    er2     ;pointers same? yes=done
    beq     playdone

    mov     @er1+,  r4l
    mov     er1,    @er0    ;save pointer back
    exts    r4              ;now ready - signed extend byte to the word

send2pwm:
    mov     r4,     @TPU1TGRB:16    ;add to the PWM

playdone:
    rts                             ;eventhough this is an interrupt
                                    ;we were called via jsr
                                    ;so do return and the caller will
                                    ;restore reg and do the rte




    .data
channel:   .long   0, 0    ;end, begin
    .end
