;
;    Cyborn OS - Copyright (C) 2004 Steve Gotthardt - Gotthardt Designs
;
;    This file is part of the Cyborn distribution.
;
;    Cyborn is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    Cyborn is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with Cyborn; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    A special exception to the GPL can be applied should you wish to distribute
;    a combined work that includes Cyborn, without being obliged to provide
;    the source code for any proprietary components.  
;

;   Since the state of the registers depends upon time, __rtcget is not
;   re-entrant


    .h8300s
    .text

    .global __rtcget
    .global __rtcinit

    
    .set PORTDDR,  0xFFFEBE
    .set PORTDATA, 0x6E
    .set PORTPINS, 0x5E
    .set SCL, 1
    .set SDA, 0

;This is a specific application of the I2C interface
; the clock is limited to 100KHz so we need to NOP to spread the time

;NOTE: do not use RMW commands on write only registers like the DDR for ports!


readbyte:
    mov     #0x02,  r1l
    mov     r1l,    @er2 ;dataline will be input
    mov     #8,     r1l         ;count the bits
nextbits:
    jsr     wait5uS
    bset    #SCL,   @PORTDATA:8
    jsr     wait5uS
    
    bld     #SDA,   @PORTPINS:8
    rotxl   r1h     ;shift the carry into our byte
    
    bclr    #SCL,   @PORTDATA:8

    dec     r1l
    bne     nextbits
    
    rts


    
;since this is timing related and I might want to overclock with a faster
;crystal ~18MHz put in a section for overclocking
wait5uS:
    nop
    nop
    nop    
    rts



;START signal
startsignal:
    bset    #SCL, @PORTDATA:8 ;clockline high

    jsr     wait5uS
    mov     #0x03,  r1l     ;make dataline output = 0
    mov     r1l,    @er2 
    
    jsr     wait5uS
        
    bclr    #SCL, @PORTDATA:8 ;clockline low
    
    rts


stopsignal:
    mov     #0x03,  r1l
    mov     r1l,    @er2
    jsr     wait5uS
    
    bset    #SCL, @PORTDATA:8 ;clockline high
    jsr     wait5uS
    
    mov     #0x02,  r1l
    mov     r1l,    @er2    ;dataline will be input
    
    rts
    
    

;uses r1l = bits to shift out
;toggle the ddr for sda to set/clr the bit
;must set ddr directly since it is write only - dont use read-modify-write inst
sendbyte:
    ;now send the bits for the device address
    ;data bit first
    mov     #3,     r1h
    mov     #8,     e1

sendbits:
    ;shift the bits to detect
    shal    r1l
    bist    #0, r1h
    mov     r1h, @er2
    
    jsr     wait5uS
    bset    #SCL, @PORTDATA:8
    
    jsr     wait5uS
    bclr    #SCL, @PORTDATA:8

    dec     #1,     e1
    bne     sendbits

    mov     #0x02,  r1h
    mov     r1h,    @er2     ;dataline will be input and go high

    jsr     wait5uS
    bset    #SCL, @PORTDATA:8
    
    jsr     wait5uS
    bclr    #SCL, @PORTDATA:8
    
    rts




__rtcinit:
    ;sda - data  - PF0 PORTF.0
    ;scl - clock - PF1 PORTF.1

    ;the data out bit is always set to a one
    ; to get a zero for an output - set the DDR to 0 for that bit

   
    mov     #PORTDDR, er2  ; Data Directon - save some bytes

    bclr    #SDA,    @er2        ;data - preset to a zero - pulldown
    bset    #SCL,    @er2        ;clock - not busy - one
    mov     #0x02,  r1l
    mov     r1l,    @er2     ;dataline will be input

    jsr     startsignal

    ;send address and write (0)
    mov     #0xA2, r0l
    jsr     sendbyte
    
    ;send address of zero
    mov     #0x00, r0l
    jsr     sendbyte

    ;send data
    mov     #0x00, r0l
    jsr     sendbyte

    jsr     stopsignal

    rts    



__rtcget:
    ;er0 is a pointer to where the data bytes will go to
    ;we are the master and start out transmitting
    ;then we will be a master receiver

    ;sda - data  - PF0 PORTF.0
    ;scl - clock - PF1 PORTF.1

   
    mov #PORTDDR, er2  ; Data Direction Port - save some bytes

    ; make the data line an output
    ; and the clock line an output
    mov     #0x02,  r1l
    mov     r1l,    @er2
    bset    #SCL,   @PORTDATA:8     ;clock - not busy - one

    ; need to wait because the processor is too fast
    jsr     wait5uS

    ; send the start signal
    jsr     startsignal

    ;send address and write
    mov     #0xA2, r1l
    jsr     sendbyte

    ;send address of one
    mov     #0x01, r1l
    jsr     sendbyte

    ; send another start - this puts the rtc into talk mode
    jsr     startsignal

    ;send address and read
    mov     #0xA3, r1l
    jsr     sendbyte

    ;read 4 registers - hundredths, seconds, minutes, hours
    mov #4, e1

    ; make the sda an input
    mov     #0x02,  r1l
    mov     r1l,    @er2     ;dataline will be input
    
    ;now read the data
nextbyte:
    jsr     readbyte    ;r1h will contain byte

    ;store the byte
    mov     r1h, @er0

    ;are we done?
    dec     #1, e1
    beq     done

    ;not done so ack with data line low
    mov     #0x03,  r1l     ;dataline as output
    mov     r1l,   @er2      

    jsr     wait5uS
    bset    #SCL,   @PORTDATA:8
    jsr     wait5uS
    bclr    #SCL,   @PORTDATA:8

    mov     #0x02,  r1l
    mov     r1l,    @er2    ;dataline goes high
 
    ; point to next byte position to write to
    inc     #1, er0
    
    jmp     nextbyte

done:
    ;send a fake ack

    ; make the sda an input
    mov     #0x02,  r1l      ;dataline input and go high
    mov     r1l,   @er2        
    bset    #SCL,  @PORTDATA:8

    bclr    #SCL,  @PORTDATA:8
    jsr     wait5uS

    jsr     stopsignal    

    rts
    


