
#include "gfx.h"
#include "font.h"
#include "pixmap.h"


typedef struct
{
    unsigned        offset;
    unsigned char   width;
} MapPositionT;


struct font
{
    FontTypeT       type;
    FontSpacingT    spacing;
    FontLeadingT    leading;
    FontHeightT     height;
    FontWidthT      width;          // width of char - max if prop
    char            first;          // first char in ASCII
    char            last;           // last char in ASCII
    PixmapT         font;           // Pixmap of font
    MapPositionT    position[];     // contains multiple widths if proportional
};



#if 0
// Load the font from someplace
FontT
FontLoad(char *from)
{
    FontT *font;

    font = malloc(sizeof(struct font));
    if (font)
    {
        // load the font info which is common to ex and internal formats

        // get memory for font - if it fails release all memory and exit

        // convert from external format to internal format - image
        font->type =
        font->spacing = FONT_SPACING_DEFAULT;
        font->leading = FONT_LEADING_DEFAULT;
        font->height =
        font->width =
        font->first =
        font->last =
        // figure positions

        //
    }

    return font;
}
#endif


// returns the maximum width for proportional, or the width of each
FontWidthT
FontWidth(FontT font)
{
    return font->width;
}



// returns max height. TODO: decenders? etc...?
FontHeightT
FontHeight(FontT font)
{
    return font->height;
}



// returns how many pixels wide the given string will be
unsigned
FontWide(FontT font, char *str)
{
    // When FontDisplay is called with 0 for context nothing is displayed and
    // only the total width is returned
    return FontDisplay(0, 0, 0, font, str);
}




void
FontFixed(FontT font)
{
    if (font->type & FONT_TYPE_PROP)
    {
        font->type |= FONT_TYPE_FFIX;
    }
}




// if the font is gray scale, the context color is ignored
unsigned
FontDisplay(GfxContextT *context, unsigned x, unsigned y, FontT font, char *str)
{
    unsigned wide;

    wide = 0;

    // only check a valid font
    if (font)
    {
        RectT rect;

        // preload these since they do not change
        rect.h = font->height;
        rect.y = 0;

        // display until the NULL terminator
        while (*str)
        {
            // only count valid chars
            if (*str >= font->first && *str<= font->last)
            {
                char  ascii;

                ascii = *str - font->first;

                // must be prop - and NOT forced fixed
                // TODO: optimize
                if ((font->type & FONT_TYPE_PROP) &&
                     !(font->type & FONT_TYPE_FFIX))
                {
                    rect.x = font->position[ascii].offset;
                    rect.w = font->position[ascii].width;
                }
                else
                {
                    // fixed width TODO: optimize
                    rect.x = font->width * ascii;
                    rect.w = font->width;  // shouldn't change if fixed
                }

                // valid display context?
                if (context)
                {
                    // finally put the font to the display
                    PixmapBlit(context, x + wide, y, &font->font, &rect);
                }

                // move the width according to char displayed
                wide += rect.w;

                // add in the spacing
                wide += font->spacing;
            }

            // next character
            str++;
        }
    }

    // how many pixels wide is the given string
    return wide;
}





// set the spacing between each line of text - default = 1
void
FontLeading(FontT font, FontLeadingT leading)
{
    font->leading = leading;
}



// set the spacing between each character - default = 0
void
FontSpacing(FontT font, FontSpacingT spacing)
{
    font->spacing = spacing;
}



/////////////////////////////////////////////////////////////////////////////


