



#include "gfx.h"


void GfxCircle(SurfaceT *surface, GfxPT xc, GfxPT yc, unsigned radius)

{
    int x, y, d;

    d = 3 - (radius<<1);
    x = 0;
    y = radius;

    while (x <= y)
    {
        GfxPixel(surface, xc+x, yc+y);
        GfxPixel(surface, xc-x, yc+y);
        GfxPixel(surface, xc+y, yc+x);
        GfxPixel(surface, xc+y, yc-x);
        GfxPixel(surface, xc+x, yc-y);
        GfxPixel(surface, xc-x, yc-y);
        GfxPixel(surface, xc-y, yc+x);
        GfxPixel(surface, xc-y, yc-x);

        if (d < 0)
        {
            d += (x << 2) + 6;
        }
        else
        {
            d += ((x-y) << 2) + 10;
            y--;
        }
        x++;
    }
}



void GfxFillCircle(SurfaceT *surface, GfxPT xc, GfxPT yc, unsigned radius)
{
    int x, y, d;

    d = 3 - (radius<<1);
    x = 0;
    y = radius;

    while (x <= y)
    {
        GfxHLine(surface, xc + x, yc + y, xc - x);
        GfxHLine(surface, xc - y, yc + x, xc + y);
        GfxHLine(surface, xc - y, yc - x, xc + y);
        GfxHLine(surface, xc - x, yc - y, xc + x);

        if (d < 0)
        {
            d +=  (x << 2) + 6;
        }
        else
        {
            d +=  ((x-y) << 2) + 10;
            y--;
        }
        x++;
    }
}



/*
  The most efficient way to calculate the ellipse positions
  is using the knowledge that the ellipse is just circle which has
  compressed (or stretched) in one direction. For a circle, the
  following equation holds true for all points located on the border of
  it:
  x^2 + y(x)^2 = r^2 = const

  Therefore, for an ellipse we can make use of the following equation:

  (ry*x)^2 + (rx*y(x))^2 = (ry*rx)^2 = const

*/
void GfxEllipse(SurfaceT *surface, GfxPT x0, GfxPT y0, unsigned rx, unsigned ry)
{
    long OutConst, Sum, SumY;
    int x,y;
    int xOld;
    long _rx = rx;
    long _ry = ry;
    OutConst = _rx*_rx*_ry*_ry  /* Constant as explained above */
        +(_rx*_rx*_ry>>1); /* To compensate for rounding */
    xOld = x = rx;
    for (y=0; y<=ry; y++)
    {
        if (y == ry)
        {
            x=0;
        }
        else
        {
            SumY =((long)(rx*rx))*((long)(y*y));
            for (;;)
            {
                Sum = SumY + ((long)(ry * ry)) * ((long)(x * x));
                if ((x <= 0) || (Sum <= OutConst)) break;
                x--;
            }
        }

        if (y)
        {
            GfxLine(surface, x0-xOld, y0-y+1, x0-x, y0-y);
            GfxLine(surface, x0-xOld, y0+y-1, x0-x, y0+y);
            GfxLine(surface, x0+xOld, y0-y+1, x0+x, y0-y);
            GfxLine(surface, x0+xOld, y0+y-1, x0+x, y0+y);
        }

        xOld = x;
    }
}



void GfxFillEllipse(SurfaceT *surface, GfxPT x0, GfxPT y0, unsigned rx, unsigned ry)
{
    long OutConst, Sum, SumY;
    int x,y;
    long _rx = rx;
    long _ry = ry;

    OutConst = _rx*_rx*_ry*_ry  /* Constant as explained above */
        +(_rx*_rx*_ry>>1);      /* To compensate for rounding */

    x = rx;
    for (y=0; y<=ry; y++)
    {
        SumY =((long)(rx*rx))*((long)(y*y));

        for (;;)
        {
            Sum = SumY + ((long)(ry * ry)) * ((long)(x * x));

            if ((x <= 0) || (Sum <= OutConst)) break;
            x--;
        }

        GfxHLine(surface, x0-x, y0+y, x0+x);
        if (y) GfxHLine(surface, x0-x, y0-y, x0+x);

    }
}


