//
//  Cyborn OS - Copyright (C) 2004 Steve Gotthardt - Gotthardt Designs
//
//  This file is part of the Cyborn distribution.
//
//  Cyborn is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  Cyborn is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with Cyborn; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  A special exception to the GPL can be applied should you wish to distribute
//  a combined work that includes Cyborn, without being obliged to provide
//  the source code for any proprietary components.
//

#include "gfx.h"
#include "rect.h"




#define     SEG_1   0x0001
#define     SEG_2   0x0002
#define     SEG_3   0x0004
#define     SEG_4   0x0008
#define     SEG_5   0x0010
#define     SEG_6   0x0020
#define     SEG_7   0x0040
#define     SEG_8   0x0080
#define     SEG_9   0x0100
#define     SEG_A   0x0200
#define     SEG_B   0x0400
#define     SEG_C   0x0800
#define     SEG_D   0x1000
#define     SEG_E   0x2000



static const unsigned short digits[] =
{
    // *
    SEG_2 | SEG_4 | SEG_6 |  SEG_8 | SEG_A | SEG_C,

    // +
    SEG_4 | SEG_A | SEG_D | SEG_E,

    // ,
    SEG_8,

    // -
    SEG_4 | SEG_A,

    // .
    SEG_E,

    // /-slash
    SEG_2 | SEG_8,

    // 0
    SEG_1 | SEG_2 | SEG_3 | SEG_5 | SEG_7 | SEG_8 | SEG_9 | SEG_B,

    // 1
    SEG_D | SEG_E,

    // 2
    SEG_1 | SEG_3 | SEG_4 | SEG_7 | SEG_8,

    // 3
    SEG_1 | SEG_3 | SEG_4 | SEG_5 | SEG_7,

    // 4
    SEG_3 | SEG_4 | SEG_5 | SEG_A | SEG_B,

    // 5
    SEG_1 | SEG_6 | SEG_7 | SEG_A | SEG_B,

    // 6
    SEG_1 | SEG_4 | SEG_5 | SEG_7 | SEG_9 | SEG_A | SEG_B,

    // 7
    SEG_1 | SEG_3 | SEG_5,

    // 8
    SEG_1 | SEG_3 | SEG_4 | SEG_5 | SEG_7 | SEG_9 | SEG_A | SEG_B,

    // 9
    SEG_1 | SEG_3 | SEG_4 | SEG_5 | SEG_7 | SEG_A | SEG_B,
};






static const unsigned short segments[] =
{
    // A
    SEG_1 | SEG_3 | SEG_5 | SEG_9 | SEG_B | SEG_A | SEG_4,

    // B
    SEG_1 | SEG_3 | SEG_4 | SEG_5 | SEG_7 | SEG_D | SEG_E,

    // C
    SEG_1 | SEG_7 | SEG_9 | SEG_B,

    // D
    SEG_1 | SEG_3 | SEG_5 | SEG_7 | SEG_D | SEG_E,

    // E
    SEG_1 | SEG_4 | SEG_7 | SEG_9 | SEG_A | SEG_B,

    // F
    SEG_1 | SEG_9 | SEG_A | SEG_B,

    // G
    SEG_1 | SEG_4 | SEG_5 | SEG_7 | SEG_9 | SEG_B,

    // H
    SEG_3 | SEG_4 | SEG_5 | SEG_9 | SEG_A | SEG_B,

    // I
    SEG_1 | SEG_7 | SEG_D | SEG_E,

    // J
    SEG_3 | SEG_5 | SEG_7 | SEG_9,

    // K
    SEG_2 | SEG_6 | SEG_9 | SEG_A | SEG_B,

    // L
    SEG_7 | SEG_9 | SEG_B,

    // M
    SEG_2 | SEG_3 | SEG_5 | SEG_9 | SEG_B | SEG_C,

    // N
    SEG_3 | SEG_5 | SEG_6 | SEG_9 | SEG_B | SEG_C,

    // O
    SEG_1 | SEG_3 | SEG_5 | SEG_7 | SEG_9 | SEG_B,

    // P
    SEG_1 | SEG_3 | SEG_4 | SEG_9 | SEG_A | SEG_B,

    // Q
    SEG_1 | SEG_3 | SEG_5 | SEG_6 | SEG_7 | SEG_9 | SEG_B,

    // R
    SEG_1 | SEG_3 | SEG_4 | SEG_6 | SEG_9 | SEG_A | SEG_B,

    // S
    SEG_1 | SEG_4 | SEG_5 | SEG_7 | SEG_A | SEG_B,

    // T
    SEG_1 | SEG_D | SEG_E,

    // U
    SEG_3 | SEG_5 | SEG_7 | SEG_9 | SEG_B,

    // V
    SEG_2 | SEG_8 | SEG_9 | SEG_B,

    // W
    SEG_3 | SEG_5 | SEG_6 | SEG_8 | SEG_9 | SEG_B,

    // X
    SEG_2 | SEG_6 | SEG_8 | SEG_C,

    // Y
    SEG_2 | SEG_C | SEG_E,

    // Z
    SEG_1 | SEG_2 | SEG_7 | SEG_8
};




// put the requested character into the rect
void
GfxFont1(SurfaceT *surface, unsigned char data, RectT *rect)
{
    unsigned short index;

#ifdef ALTERNATE_METHOD // maybe faster?
#define x1  rect->x
#define y1  rect->y
#define x2  (rect->x + rect->w - 1)
#define y2  (rect->y + rect->h - 1)
#define xm  ((x1) + (x2)) >> 1)
#define ym  ((y1) + (y2)) >> 1)
#else
    GfxPT x1 = rect->x;
    GfxPT y1 = rect->y;
    GfxPT x2 = rect->x + rect->w - 1;
    GfxPT y2 = rect->y + rect->h - 1;
    GfxPT xm = (x1 + x2) >> 1;
    GfxPT ym = (y1 + y2) >> 1;
#endif

    // turn the data into an index
    if (data >= 'A' && data <= 'Z')
    {
        index = segments[data - 'A'];
    }
    else if (data >= '*' && data <= '9')
    {
        index = digits[data - '*'];
    }
    else
    {
        // dont know how to do this one...
        return;
    }


    if (index & SEG_1)
    {
        // across the top
        GfxHLine(surface, x1, y1, x2);
    }

    if (index & SEG_2)
    {
        // upper right to middle
        GfxLine(surface, xm, ym, x2, y1);
    }

    if (index & SEG_3)
    {
        // upper right segment
        GfxVLine(surface, x2, y1, ym);
    }

    if (index & SEG_4)
    {
        // middle right
        GfxHLine(surface, xm, ym, x2);
    }

    if (index & SEG_5)
    {
        // bottom right segment
        GfxVLine(surface, x2, ym, y2);
    }

    if (index & SEG_6)
    {
        // lower right to middle
        GfxLine(surface, xm, ym, x2, y2);
    }

    if (index & SEG_7)
    {
        // across the bottom
        GfxHLine(surface, x1, y2, x2);
    }

    if (index & SEG_8)
    {
        // lower left to middle
        GfxLine(surface, x1, y2, xm, ym);
    }

    if (index & SEG_9)
    {
        // bottom left segment
        GfxVLine(surface, x1, ym, y2);
    }

    if (index & SEG_A)
    {
        // middle left segment
        GfxHLine(surface, x1, ym, xm);
    }

    if (index & SEG_B)
    {
        // middle left segment
        GfxVLine(surface, x1, y1, ym);
    }

    if (index & SEG_C)
    {
        // upper left to middle
        GfxLine(surface, xm, ym, x1, y1);
    }

    if (index & SEG_D)
    {
        // middle upper segment
        GfxVLine(surface, xm, y1, ym);
    }

    if (index & SEG_E)
    {
        // lower upper segment
        GfxVLine(surface, xm, ym, y2);
    }


}


