#include "gfx.h"
#include "types.h"


typedef struct
{
    Bit0:1;
    Bit1:1;
    Bit2:1;
    Bit3:1;
    Bit4:1;
    Bit5:1;
    Bit6:1;
    Bit7:1;
} BitsT;


typedef struct
{
    UINT8   width;
    UINT8   height
    BitsT   *map;
} BitmapT;


typedef struct
{
    UINT8   wide;   // number of pixels wide
    UINT8   high;   // number of pixels high
    UINT8   xoff;   // offset to start of x data
    UINT8   yoff;   // offset to start of y data - row
    UINT8   data[];
} GlyphT;




void
GlyphBlit(SurfaceT *surface, GlyphT *glyph, UINT16 x, UINT16 y)
{
    UINT8 *startbyte;
    UINT8 *surfacebyte;

    startbyte = surface->data + surface->cols * y + x/4;

    if (glyphbyte & glyphmask)
    {
        *surfacebyte |= surfacemask & surface->foreground;
    }
    else
    {
        *surfacebyte |= surfacemask & surface->background;
    }

    surfacemask >>= 2;
    if (surfacemask == 0)
    {
        surfacemask = 0xC0;
        surfacebyte++;
    }


    glyphmask >>= 1;
    if (glyphmask == 0)
    {
        glyphmask = 0x80;
        glyphbyte++;
    }

    // mask
    *startbyte = *startbyte

}


UINT   // returns width of chars placed
GfxFontRender(SurfaceT *surface, const FontT font, UCHAR *data, RectT *rect)
{
    UINT    xstart;
    UINT8   *indexbyte, *linebyte, *glyphbyte;


    switch (font->type)
    {
        case FONT_TYPE_FAST:
        rect->x &= ~3;  // align to even byte start
        xstart = rect->x;
        indexbyte = surface->buffer + (surface->cols * rect->y) + (xstart / 4);
        while (*data)
        {
            GlyphT *glyph;

            if (*data >= font->first && *data <= font->last)
            {
                UINT8 height;

                // pull out the glyph
                glyph = font->glyph + (*data - font->first);
                glyphbyte = glyph->data;

                // blit the glyph onto the surface - we destructively copy
                height = 0;   // - glyph->height;
                linebyte = indexbyte;
                while (height < font->hmax)
                {
                    if (height < glyph->offset)
                    {
                        memset(linebyte, surface->background, glyph->width);
                    }
                    else
                    {
                        // for fast fonts width is in bytes
                        UINT8 width = glyph->width;
                        UINT8 *targetbyte = linebyte;

                        while (width--)
                        {
                            if (*glyphbyte)
                            {
                                *targetbyte++ =
                                    (*glyphbyte & surface->foreground) ||
                                    (~*glyphbyte & surface->background);
                            }
                            else
                            {
                                *targetbyte++ = surface->background;
                            }

                            glyphbyte++;
                        }
                    }
                    linebyte += surface->cols;
                }

                // move x along to where the next char would go
                rect->x += glyph->width * 8;
            }

            // next char to render
            data++;

            // move the index along across the line
            indexbyte += font->maxw;
        }
        break;


        case FONT_TYPE_BITMAP:
        xstart = rect->x;
        while (data)
        {
            GlyphT *glyph;
            if (data >= font->first && data <= font->last)
            {
                // pull out the glyph
                glyph = font->glyph + (data - font->start);

                // blit the glyph onto the surface
                GlyphBlit(surface, glyph, rect->x, rect->y);

                // move x along to where the next char would go
                rect->x += (glyph->width + 1 /* + spacing? */);
            }

            // next char to render
            data++;
        }
        break;

        // This uses the built in scalable font
        case FONT_TYPE_SCALEABLE:
        xstart = rect->x;
        while (*data)
        {
            if (*data >= font->first && *data <= font->last)
            {
                // This is a scalable font
                GfxFont1(surface, *data, rect);

                // add width and character spacing
                rect->x += rect->w + 1;
            }

            // next char to render
            data++;
        }
        break;
    }

    // calculate the width used
    return rect->x - xstart;

}

