
#include "gfx.h"
//#include "rect.h"



typedef struct
{
    UINT8 pixel0:2;
    UINT8 pixel1:2;
    UINT8 pixel2:2;
    UINT8 pixel3:2;
} PixelsT;






// This is a very intense function - trying to be as fast as possible using
// 4 different cases where the bitmap will be aligned with the surface

// non-transparent blit
void
GfxPixmapBlit(SurfaceT *surface, const PixelmapT *pixelmap, unsigned x, unsigned y)
{
    PixelsT *surfacerow;
    PixelsT *pixelbuffer;
    UINT8 *pbuffer;
    UINT8 pixels, height;


    // per blit
    surfacerow  = (PixelsT *)surface->buffer + (y * surface->cols) + (x>>2);
    pixelbuffer = (PixelsT *)pixelmap->image;

    pbuffer = pixelmap->image;;
    height = pixelmap->height;

    // for each row offset
    switch (x & 3)
    {
        case 0:  // aligned

            // do all the rows
            while (height--)
            {
                pixels = pixelmap->width;
                UINT8 *sbuffer = (UINT8 *)surfacerow;

                // do some heavy lifting
                while (pixels >= 4)
                {
                    pixels -= 4;
                    *sbuffer++ = *pbuffer++;
                }

                // do the right end - if there is one
                if (pixels)
                {
                    *sbuffer =
                        (*pbuffer   & ~gfxleftmask[pixels]) |
                        (*sbuffer & gfxleftmask[pixels]);

                    pbuffer++;
                }

                // jump to next place in surface for the next row
                surfacerow += surface->cols;
            }
            break;


        case 1:  // off by one
            while (height--)
            {
                // for one row of pixels
                PixelsT *surfacebuffer = (PixelsT *)surfacerow;
                pixels=pixelmap->width;

                while (pixels >= 4)
                {
                    pixels -= 4;

                    *(UINT8 *)surfacebuffer = (*(UINT8 *)surfacebuffer & 0xC0)
                        | (*(UINT8 *)pixelbuffer >> 2);

                    surfacebuffer++;
                    surfacebuffer->pixel0 = pixelbuffer->pixel3;
                    pixelbuffer++;
                }

                if (pixels)
                {
                    surfacebuffer->pixel1 = pixelbuffer->pixel0;

                    if (pixels > 1)
                    {
                        surfacebuffer->pixel2 = pixelbuffer->pixel1;
                        if (pixels > 2)
                        {
                            surfacebuffer->pixel3 = pixelbuffer->pixel2;
                        }
                    }
                    pixelbuffer++;
                }

                // one 'width' is done - go to front
                surfacerow += surface->cols;
            }
            break;


        case 2:
            while (height--)
            {
                PixelsT *surfacebuffer = surfacerow;

                // for one row of pixels
                // must go pixel by pixel (2 bits at a time)
                for (pixels=pixelmap->width;;)
                {
                    // clear pixel in surface and OR the pixel from sprite
                    surfacebuffer->pixel2 = pixelbuffer->pixel0;
                    if (--pixels == 0) break;

                    surfacebuffer->pixel3 = pixelbuffer->pixel1;
                    if (--pixels == 0) break;

                    surfacebuffer++;

                    surfacebuffer->pixel0 = pixelbuffer->pixel2;
                    if (--pixels == 0) break;

                    surfacebuffer->pixel1 = pixelbuffer->pixel3;
                    if (--pixels == 0) break;

                    pixelbuffer++;
                }

                pixelbuffer++;  // one row done
                surfacerow += surface->cols;
            }
            break;

        case 3:
            while (height--)
            {
                PixelsT *surfacebuffer = surfacerow;
                pixels=pixelmap->width;

                while (pixels >= 4)
                {
                    pixels -= 4;
                    surfacebuffer->pixel3 = pixelbuffer->pixel0;
                    surfacebuffer++;

                    *(UINT8 *)surfacebuffer = (*(UINT8 *)surfacebuffer & 0x03)
                        | (*(UINT8 *)pixelbuffer << 2);

                    pixelbuffer++;
                }

                if (pixels)
                {
                    surfacebuffer->pixel3 = pixelbuffer->pixel0;

                    if (pixels > 1)
                    {
                        surfacebuffer++;
                        surfacebuffer->pixel0 = pixelbuffer->pixel1;

                        if (pixels > 2)
                        {
                            surfacebuffer->pixel1 = pixelbuffer->pixel2;
                        }
                    }
                    pixelbuffer++;
                }
                surfacerow += surface->cols;
            }
            break;
    }

}



#if 0
// saved
                PixelsT *surfacebuffer = surfacerow;
                // must go pixel by pixel (2 bits at a time) - for ONE row
                for (pixels=pixelmap->width;;)
                {
                    // clear pixel in surface and OR the pixel from sprite
                    surfacebuffer->pixel3 = pixelbuffer->pixel0;
                    if (--pixels == 0) break;

                    surfacebuffer++;

                    surfacebuffer->pixel0 = pixelbuffer->pixel1;
                    if (--pixels == 0) break;

                    surfacebuffer->pixel1 = pixelbuffer->pixel2;
                    if (--pixels == 0) break;

                    surfacebuffer->pixel2 = pixelbuffer->pixel3;
                    //asm volatile (" bld %0, @%1" :: "i" (0), "r" (pixelbuffer));
                    //asm volatile (" bst %0, @%1" :: "i" (1), "r" (surfacebuffer));
                    //asm volatile (" bld %0, @%1" :: "i" (2), "r" (pixelbuffer));
                    //asm volatile (" bst %0, @%1" :: "i" (3), "r" (surfacebuffer));
                    if (--pixels == 0) break;

                    pixelbuffer++;
                }

                pixelbuffer++;  // one row done
                surfacerow += surface->cols;




// if each byte represented a pixel, this function would be small. But faster?
void
Gfx8PixmapBlit(SurfaceT *surface, PixelmapT *pixelmap, unsigned x, unsigned y)
{
    unsigned char *surfacebuffer;
    unsigned char *pixelbuffer;
    unsigned height;

    // per blit
    surfacebuffer = surface->buffer + (y * surface->cols) + x;
    pixelbuffer = pixelmap->image;
    height = pixelmap->height;

    // per row
    while (height--)
    {
        memcpy(surfacebuffer, pixelbuffer, pixelmap->width);
        pixelbuffer += pixelmap->width;
        surfacebuffer += surface->cols;
    }

}

#endif
