
#include "gfx.h"
#include "rect.h"



// use built in maps in the ROM
static const unsigned char masks[] = { 0xC0, 0x30, 0x0C, 0x03 };


// non-transparent blit
void
GfxSpriteBlit(SurfaceT *surface, SpriteT *sprite, unsigned x, unsigned y)
{
    unsigned char pixelmask, surfacemask, pixeldata, surfacedata;
    unsigned char *surfacebuffer;
    unsigned char *pixelbuffer;
    unsigned pixel;
    //

    surfacebuffer = surface->buffer + y * surface->cols + x/4;
    pixelbuffer   = sprite->pixelmap->buffer;

    // for each row and column in the sprite pixelmap
    // slide along with the x value


// for each row
    switch (x&3)
    {
        case 0:  // aligned
        for (row = 0; row<sprite->height; row++)
        {
            // the memcpy is a luxery for byte aligned surface and sprite
            memcpy(surfacebuffer, pixelbuffer, sprite->width / 4);
            surfacebuffer += sprite->pixelmap->stride +1;

            // if there are extra pixels, do them here:
            switch (sprite->width & 0x03)
            {
                case 1:
                    // do the ragged right side of the sprite
                    *surfacebuffer &= 0x3F;
                    *surfacebuffer |= (*pixelbuffer & 0xC0);
                    break;

                case 2:
                    // do the ragged right side of the sprite
                    *surfacebuffer &= 0x0F;
                    *surfacebuffer |= (*pixelbuffer & 0xF0);
                    break;

                case 3:
                    // do the ragged right side of the sprite
                    *surfacebuffer = (*surfacebuffer & 0x03) | (*pixelbuffer & 0xFC);
                    break;

                default:
            }

            // ready for the next row
            surfacebuffer += surface->cols;
            pixelbuffer += sprite->stride;


            if (pixeldata & 0xC0)
            {
                // need to put the pixel into the surface
                *surfacebuffer =
                    (*surfacebuffer & 0x3F) | (*pixelbuffer & 0xC0);
            }

            if (pixeldata & 0x30)
            {
                // need to put the pixel into the surface
                *surfacebuffer =
                    (*surfacebuffer & 0xCF) | (*pixelbuffer & 0x30);
            }

            if (pixeldata & 0x0C)
            {
                // need to put the pixel into the surface
                *surfacebuffer =
                    (*surfacebuffer & 0xF3) | (*pixelbuffer & 0x0C);
            }

            if (pixeldata & 0x03)
            {
                // need to put the pixel into the surface
                *surfacebuffer =
                    (*surfacebuffer & 0xFC) | (*pixelbuffer & 0x03);
            }

            // point to the next byte
            pixelbuffer++;
            surfacebuffer++;
        }
            break;


        case 1:  // off by one

            // must go pixel by pixel (2 bits at a time) - this is for ONE row
            for (pixel=sprite->width;;)
            {
                pixeldata = *pixelbuffer >> 2;
                surfacedata = *surfacebuffer;

                // clear pixel in surface and OR the pixel from sprite
                *surfacebuffer = *surfacebuffer & ~0x30 | pixeldata & 0x30;
                if (--pixel == 0) break;


                *surfacebuffer = *surfacebuffer & ~0x0C | pixeldata & 0x0C;
                if (--pixel == 0) break;

                *surfacebuffer = *surfacebuffer & ~0x03 | pixeldata & 0x03;
                if (--pixel == 0) break;

                pixeldata = *pixelbuffer << 6;
                surfacebuffer++;

                *surfacebuffer = *surfacebuffer & ~0xC0 | pixeldata;
                if (--pixel == 0) break;

                pixelbuffer++;
            }
            break;

        case 2:
            // must go pixel by pixel (2 bits at a time)
            for (pixel=0;;)
            {
                pixeldata = *pixelbuffer >> 4;
                surfacedata = *surfacebuffer;

                // clear pixel in surface and OR the pixel from sprite
                *surfacebuffer = *surfacebuffer & ~0x0C | pixeldata & 0x0C;
                if (++pixel > sprite->width) break;

                *surfacebuffer = *surfacebuffer & ~0x03 | pixeldata & 0x03;
                if (++pixel > sprite->width) break;

                pixeldata = *pixelbuffer << 4;
                surfacebuffer++;

                *surfacebuffer = *surfacebuffer & ~0xC0 | pixeldata & 0xC0;
                if (++pixel > sprite->width) break;

                *surfacebuffer = *surfacebuffer & ~0x30 | pixeldata & 0x30;
                if (++pixel > sprite->width) break;

                pixelbuffer++;
            }

            break;

        case 3:
            // must go pixel by pixel (2 bits at a time) - this is for ONE row
            for (pixel=sprite->width;;)
            {
                pixeldata = *pixelbuffer >> 6;
                surfacedata = *surfacebuffer;

                // clear pixel in surface and OR the pixel from sprite
                *surfacebuffer = *surfacebuffer & ~0x03 | pixeldata;
                if (--pixel == 0) break;

                pixeldata = *pixelbuffer << 2;
                surfacebuffer++;

                *surfacebuffer = *surfacebuffer & ~0xC0 | pixeldata & 0xC0;
                if (--pixel == 0) break;

                *surfacebuffer = *surfacebuffer & ~0x30 | pixeldata & 0x30;
                if (--pixel == 0) break;

                *surfacebuffer = *surfacebuffer & ~0x0C | pixeldata & 0x0C;
                if (--pixel == 0) break;

                pixelbuffer++;
            }

            break;

    }


    pixdata = *pixelbuffer & gfxpixelmask;
    surfacedata = surfacemask & surfacebuffer;

    sprite->
    switch (map->layers)
    {
        char temp;

        // true bitmap
        case 1:
            if (map->image[y * map->stride + (x >> 3)] & (0x80 >> (x&7)))
            {
                return GFX_COLOR_BLK;
            }
            else
            {
                return GFX_COLOR_WHT;
            }
            break;

        // grayscale - four color - 2 bits
        case 2:
            temp = map->image[y * map->stride + (x >> 2)] & (0xC0 >> (x&3));
            switch (x&3)
            {
                case 0: return temp >> 6;
                case 1: return temp >> 4;
                case 2: return temp >> 2;
                case 3: return temp;
            }
            break;
    }
}



void
PixmapBlit(GfxContextT  *context,   // graphics context
        unsigned        x,          // where to put the image
        unsigned        y,          //
        PixmapT         *src,       // pixmap source
        RectT           *rect)      // destination location
{
    unsigned destx;
    unsigned desty;

    // TODO: use monitor, etc to optimize this
    for (destx = 0; destx < rect->w; destx++)
    {
        for (desty = 0; desty < rect->h; desty++)
        {
            switch (PixmapColor(src, rect->x + destx, rect->y + desty))
            {
                case GFX_COLOR_BLK:
                    GfxPixelX(context, x + destx, y + desty, GFX_COLOR_BLK);
                    break;

                case GFX_COLOR_LTG:
                    GfxPixelX(context, x + destx, y + desty, GFX_COLOR_LTG);
                    break;

                case GFX_COLOR_DKG:
                    GfxPixelX(context, x + destx, y + desty, GFX_COLOR_DKG);
                    break;

                // check if PUT or OR - PUT is the assumed state
                case GFX_COLOR_WHT:
                    if ((context->mode & GFX_MODE_OR) == 0)
                    {
                        GfxPixelX(context, x + destx, y + desty, GFX_COLOR_WHT);
                    }
                    break;
            }
        }
    }
}


