
#include "gfx.h"
#include "rect.h"



// what color is at the (x, y)?
unsigned
PixmapColor(PixmapT *map, unsigned x, unsigned y)
{
    switch (map->layers)
    {
        char temp;

        // true bitmap
        case 1:
            if (map->image[y * map->stride + (x >> 3)] & (0x80 >> (x&7)))
            {
                return GFX_COLOR_BLK;
            }
            else
            {
                return GFX_COLOR_WHT;
            }
            break;

        // grayscale - four color - 2 bits
        case 2:
            temp = map->image[y * map->stride + (x >> 2)] & (0xC0 >> (x&3));
            switch (x&3)
            {
                case 0: return temp >> 6;
                case 1: return temp >> 4;
                case 2: return temp >> 2;
                case 3: return temp;
            }
            break;
    }
}



void
PixmapBlit(GfxContextT  *context,   // graphics context
        unsigned        x,          // where to put the image
        unsigned        y,          //
        PixmapT         *src,       // pixmap source
        RectT           *rect)      // destination location
{
    unsigned destx;
    unsigned desty;

    // TODO: use monitor, etc to optimize this
    for (destx = 0; destx < rect->w; destx++)
    {
        for (desty = 0; desty < rect->h; desty++)
        {
            switch (PixmapColor(src, rect->x + destx, rect->y + desty))
            {
                case GFX_COLOR_BLK:
                    GfxPixelX(context, x + destx, y + desty, GFX_COLOR_BLK);
                    break;

                case GFX_COLOR_LTG:
                    GfxPixelX(context, x + destx, y + desty, GFX_COLOR_LTG);
                    break;

                case GFX_COLOR_DKG:
                    GfxPixelX(context, x + destx, y + desty, GFX_COLOR_DKG);
                    break;

                // check if PUT or OR - PUT is the assumed state
                case GFX_COLOR_WHT:
                    if ((context->mode & GFX_MODE_OR) == 0)
                    {
                        GfxPixelX(context, x + destx, y + desty, GFX_COLOR_WHT);
                    }
                    break;
            }
        }
    }
}


