
// Stand alone flash program for the classic cybiko

#include "string.h"

void cyConsole(char *, long);



#define FLASH_BASE 0x100000



// SST makes the flash memory in the cybiko
#define MFG_ID_SST          0xBF

#define SST_ID_SST39VF512   0xD4
#define SST_ID_SST39VF010   0xD5
#define SST_ID_SST39VF020   0xD6
#define SST_ID_SST39VF040   0xD7




// The data will be converted to a file of chars that this include
//// will read into an array
char flashdata[] =
{
    #include "c.h"
};


inline
void
Delay100mS(void)
{
    unsigned long delay = 0x25555;

    // waiting...
    while (delay--) ;
}



char *
FlashId(unsigned long *size)
{
    char device;
    char mfg;

    // enter the Software ID mode
    *(volatile char *)(FLASH_BASE + 0x5555) = 0xAA;
    *(volatile char *)(FLASH_BASE + 0x2AAA) = 0x55;
    *(volatile char *)(FLASH_BASE + 0x5555) = 0x90;

    mfg     = *(char *)(FLASH_BASE + 0x0000);
    device  = *(char *)(FLASH_BASE + 0x0001);

    // exit the Software ID mode
    *(volatile char *)(FLASH_BASE + 0x0000) = 0xF0;

    // now figure out what the device is
    if (mfg == MFG_ID_SST)
    {
        switch (device)
        {
            case SST_ID_SST39VF512:
                *size = 0x10000;            // 64K x 8
                return "SST SST39VF512";

            case SST_ID_SST39VF010:
                *size = 0x20000;            // 128K x 8
                return "SST SST39VF010";

            case SST_ID_SST39VF020:
                *size = 0x40000;            // 256K x 8
                return "SST SST39VF020";

            case SST_ID_SST39VF040:
                *size = 0x80000;            // 512K x 8
                return "SST SST39VF040";

            default:
                return "SST unknown";
        }
    }

    // unknown manufacturer
    return NULL;
}



// ASSUMES: in program mode!
inline
void
FlashWait(volatile char *address)
{
    // wait until the toggle bit stops toggling
    while ((*address ^ *address) & 0x40)
    {
        // waiting...
    }

}



void
FlashErase(void)
{
    // Chip erase commands - use volatile here
    *(volatile char *)(FLASH_BASE + 0x5555) = 0xAA;
    *(volatile char *)(FLASH_BASE + 0x2AAA) = 0x55;
    *(volatile char *)(FLASH_BASE + 0x5555) = 0x80;
    *(volatile char *)(FLASH_BASE + 0x5555) = 0xAA;
    *(volatile char *)(FLASH_BASE + 0x2AAA) = 0x55;
    *(volatile char *)(FLASH_BASE + 0x5555) = 0x10;

    // wait for the chip to finish erasing - 100ms
    Delay100mS();
}




int
main(int argc, char *argv[])
{
    char            *flashpointer;
    char            *datapointer;
    unsigned long   flashsize;
    unsigned long   size;

    char *string;

    // what chip is this?
    string = FlashId(&flashsize);

    // display device id and mfg id
    if (string)
    {
        cyConsole("Flash ID: %s\n", (long)string);
        cyConsole("Size is %ld\n", (long)flashsize);
        Delay100mS();
    }
    else
    {
        cyConsole("unknown device\n", (long)string);
    }

    Delay100mS();


return 0;
return 0;
return 0;
return 0;
return 0;
return 0;




    // erase all of the flash
    // display that flash is going to be released
    FlashErase();

    flashpointer = (char *)FLASH_BASE;
    datapointer  = flashdata;
    size = sizeof(flashdata);

    // display that device is being programmed
    while (size--)
    {
        // unlock the byte
        *(volatile char *)(FLASH_BASE + 0x5555) = 0xAA;
        *(volatile char *)(FLASH_BASE + 0x2AAA) = 0x55;
        *(volatile char *)(FLASH_BASE + 0x5555) = 0xA0;

        // program the byte to the flash
        *flashpointer = *datapointer++;

        FlashWait(flashpointer++);
    }



    // display that device is being verified

    if (memcmp((char *)FLASH_BASE, flashdata, sizeof(flashdata)))
    {
        // display there is a mismatch
        return 1;
    }


    return 0;
}








