#ifndef _VIEWPORT_H_
#define _VIEWPORT_H_


// get all the good stuff from the OS
#include "cyborn.h"


// viewport is a OS object

typedef struct
{
    unsigned char   *buffer;
    short           size;
    short           stride;
} FrameBufferT;


typedef struct viewport
{
    FrameBufferT    *framebuffer;   // pointer to framebuffer
    CyCT            x;              // position
    CyCT            y;
    short           refresh;        // need to be refreshed?
    struct viewport *next;          // next viewport
} ViewPortT;



#define APP_NAME_MAX 32


/////////////////////////////////////////////////////////////////
////  A P P L I C A T I O N - or what?
/////////////////////////////////////////////////////////////////
// operating system holds these and can switch from one to the other
// The main keystroke producer will send keystrokes to the application
// queue which means that other applications waiting for keystrokes will
// block (automatic!) waiting for keystrokes.
// Only the viewports for the current app will be 'run'.
// They will be run at the current 'framerate'
typedef struct application
{
    char        name[APP_NAME_MAX];
    ViewPortT   viewports;
    CQueueT     keystrokes;     // where to send keystrokes when active
    short       framerate;      // maximum rate to display at
    // resources?
//    TaskT       tasks;          // associated tasks
// environment?
} CyBornAppT;



/////////////////////////////////////////////////////////////////
////  O P E R A T I N G   S Y S T E M    -   C y B o r n
/////////////////////////////////////////////////////////////////
/*
    Startup
    Clear out all memory to be used
    Set up timers / Watchdog to do task switching and display
    Set up some kind of shell / launcher / monitor as first application
*/

typedef struct cybornOS
{
    CyBornAppT      *apps;
    CyBornAppT      *current;
    TaskT           *CurrentTask;

    // some kind of background
    // idle task
    // something owns the keyboard task
    // something owns the display driver
    // some kind of task that launches new applications - shell?
    // run levels?
    // login for users?
    // list of flags (mutex, semaphores...)
    // list of queues?


    // IO file system stuff
} CyBornOsT;



#endif

