

TaskT *CurrentTask;





typedef struct
{
    TaskT       *blocked;     // points to next blocked task
} BlockerT;



typedef struct
{
    BlockerT    blocked;    // points to next blocked task
    RawSemT     *sem;       // sem pointer
    TaskT       *owner;     // sem owner
} SemT;






void inline Unblock(BlockerT *blocker, TaskT *task)
{
    // save the next blocked task (it may be zero and there is no blocked task)
    blocker->blocked = task->blocked;

    // clear the holder
    task->blocked = 0;
}




// add different ways to block - by priority, by FIFO, by time
void inline Block(BlockerT *blocker, TaskT *task)
{
    // save the next blocked task
    task->blocked = blocker->blocked;

    // save this as the head
    blocker->blocked = task;
}




void ScheduleNow(void)
{
    TaskT *task;
    PriorityT priority;

    // go through the list(s) to find the next task to run
    // all tasks are in priority order

    // a quick and dirty way:
    // start at highest priority task and look for task ready to run
    task = TaskHead;
    priority = CurrentTask->priority;

    for (;;)
    {
        if (task->state != TASK_STATE_READY)
        {
            task = task->next;
        }
        else  // READY
        {
            if (task->priority == priority)
            {
                // record the first entry in the equivalent priority list
                TaskT *first = task;

                // start at current and look
                task = CurrentTask->next;
                    for(;;)
                    {
                        if (task->priority != priority)
                        {
                            // failed to find equal priority task
                            // go back to the first
                            task = first;
                            // now look until we are back at current
                            if (task == CurrentTask)
                        }

                        if (task->state == TASK_STATE_READY)
                        {
                            // we have a different task at same priority to run
                            break;
                        }
                    }
                }
            }
            // task holds the highest priority task that is ready to run
            // move stackpointer to H8S ER7
            // and do return from exception
        }

    }// should never get here! The idle task is always ready!
}




void inline ScheduleReady(TaskT *task)
{
    task->state = TASK_READY;
}




void inline ScheduleWait(TaskT *task)
{
    task->state = TASK_WAIT;
}





void Sm4Give(SemT *sem)
{
    int ready = SemDown(sem);

    // did a task become ready?
    if (ready)
    {
        TaskT *task = sem->blocked;

        ASSERT(task);

        Unblock(&sem->blocked, task);

        // task has just become ready - reschedule tasks
        ScheduleReady(task);

        // run new task
        ScheduleNow();

    }
}



void Sm4Take(SemT *sem)
{
    int ready = SemDown(sem);

    // oops! sem is taken
    if (!ready)
    {
        Block(&sem->blocked, CurrentTask);

        // task is blocked
        ScheduleWait(CurrentTask);

        // run new task
        ScheduleNow();
    }
    else
    {
        // last owner - do we care?
        sem->owner = CurrentTask;
    }

}




// also known as sleep
void TimerBlock(TaskT *task, unsigned ticks)
{
    // add task to timer list.

    // if ticks are zero then only change to another task of same priority
    if (ticks)
    {
        ScheduleWait(task);
    }


    ScheduleNow();
}




void sleep(unsigned ticks)
{
    TimerBlock(CurrentTask, ticks);
}





void main(int, *char[])
{
    SemT sem1, sem2, sem3;

    SemCreate(&sem1, SEM_CREATE_BINARY, SEM_CREATE_FULL);
    SemCreate(&sem2, SEM_CREATE_BINARY, SEM_CREATE_FULL);
    SemCreate(&sem3, SEM_CREATE_BINARY, SEM_CREATE_FULL);


    TaskCreate(&task1, 2000, BlinkLed);

}





