
///////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////// Bottom Half
// Bottom half processing can be done here in immediate tasks -
// The scheduling is turned off - the timer is stopped - to do these tasks



static BottomHalfT volatile *ImmediateExecution;
static CybornSemT BottomHalfWaiting;



// ISR Context
// Use it in an ISR
// This should be the last line in an isr
//  Example
#if 0
BottomHalfT BHScheduler;
BottomHalfRegister(&BHScheduler, scheduler, interruptmask);

Timer Interrupt(void)
{
    // clear interrupt

    // need to run the scheduler
    BottomHalfDo(&BHScheduler);
}
#endif



// set up the bottom half
void
BottomHalfRegister(BottomHalfT *bh, EntryT entry, void *number)
{
    bh->parameter = number;
    bh->next = 0;
    bh->entry = entry;
}


// ISR context
void inline
BottomHalfDo(BottomHalfT *dsr)
{
    // add to the head of the list
    dsr->next = ImmediateExecution;

    // add to the empty list
    ImmediateExecution = dsr;

    // tell the bottom half task to service the rest of the ISR
    // use a counting semaphore
    SignalCIsrGive(&BottomHalfWaiting);
}



// Top priority - use as entry to a task
// This works by waiting to run waiting for a signal from the bottom half
// do function.
// NOTE: The bottom half MUST be interruptable. The interrupts are on.
void
BottomHalfTask(void)
{
    BottomHalfT *bh;

    SignalInit(&BottomHalfWaiting, 0);

    for (;;)
    {
        // COUNTING SEMAPHORE
        // stay on the waiting list until signalled
        SignalWait(&BottomHalfWaiting);

///////////////////////////////////
        /////// disable interrupts

        // take an entry
        bh = ImmediateExecution;

        // take it off the list
        ImmediateExecution = ImmediateExecution->next;

        /////// enable interrupts
/////////////////////////////////

        // run the bh
        bh->entry(bh->parameter);
    }
}
/////////////////////////////////////////////////////////////// Bottom Half
///////////////////////////////////////////////////////////////////////////////

