

#include "cyborn.h"



typedef void *NoteT;



typedef struct
{
    NoteT   *head;
    NoteT   *tail;
    NoteT   *first;
    NoteT   *last;
    SemT    *sem;
    BOOL    full;
} NoteQueueT;



NoteQueueT *
NoteQueueNew(unsigned short size)
{
    NoteQueueT *q;

    // allocate memory for the queue structure
    q = malloc(sizeof(NoteQueueT));

    // allocate memory for the notes
    q->first = (NoteT *)malloc(size * sizeof(NoteT));
    q->head = q->first;
    q->tail = q->first;
    q->last = q->first + size;

    // the queue is not full
    q->full = FALSE;

    return q;
}



void
NoteQueueDelete(NoteQueueT *q)
{
    // toss the queue that holds the notes
    free(q->first);

    // toss the q structure
    free(q);
}



StatusT
NoteWrite(NoteQueueT *q, NoteT note)
{
    StatusT status;

    // dont write a note if full
    if (!q->full)
    {
        INTERRUPTS_DISABLE();

        // put the note on the queue
        *q->head++ = note;

        // go too far? wrap around to the front
        if (q->head >= q->last)
        {
            q->head = q->first;
        }

        // have we wrapped around and filled the queue?
        if (q->head == q->tail)
        {
           q->full = TRUE;
        }

        INTERRUPTS_ENABLE();

        // make any waiting task ready
        status = Signal(q->sem);
    }
    else
    {
        status = STATUS_FULL;
    }

    return status;
}



// ISR context only
// This will drop an entry if there is no room left on the q
StatusT
NoteWriteIsr(NoteQueueT *q, NoteT note)
{
    StatusT status;

    if (!q->full)
    {
        // put the note on the queue
        *q->head++ = note;

        // go too far? wrap around
        if (q->head >= q->last)
        {
            q->head = q->first;
        }

        // have we wrapped around and filled the queue?
        if (q->head == q->tail)
        {
           q->full = TRUE;
        }

        // make any waiting task ready
        status = SignalIsr(q->sem);
    }
    else
    {
        status = STATUS_FULL;
    }

    return status;
}



StatusT
NoteRead(NoteQueueT *q, NoteT &note)
{
    // wait for a note to be passed
    SignalWait(q->sem);

    INTERRUPTS_DISABLE();

    // save the note
    *note = *q->tail++;

    if (q->tail >= q->last)
    {
        q->tail = q->first;
    }

    // no longer full
    q->full = FALSE;

    INTERRUPTS_ENABLE();

    return STATUS_OK;
}







