#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <netinet/in.h>



// Program target arch: x86
// Font input file is big endian format for shorts



#define IS_FONT_SET(image, x, y, stride) \
    (image[((x)>>3) + (y) * (stride)] & (0x80 >> ((x)&7)))


//////////////////////////////////////  C Y B I K O - O S   F O N T
//////////////////////////////////////////  F O R M A T S
// header for each and every bitmap
typedef struct
{
    u_int8_t    Hoff; // horizontal offset
    u_int8_t    Voff; // vertical offset
    u_int8_t    Wide; // how wide is the bitmap
    u_int8_t    High; // how high
} BitmapHeaderT;



// found at the head of the file
typedef struct
{
    u_int8_t    Type;       // what have we got in here?
    u_int8_t    Count;      // how many bitmaps?
    u_int8_t    Wmax;       // widest bitmap
    u_int8_t    Hmax;       // highest bitmap
} BitmapSequenceT;


struct
{
    BitmapHeaderT   header;
    u_int8_t        empty;
} EmptyBitmap = { { 0, 0, 1, 1 }, 0 };



// convert font to cyborn / cybiko format


// NFNT format
// FON/FNT format

//////// Font header for a NFNT font 
struct
{
    u_int16_t   type;
    u_int16_t   first;
    u_int16_t   last;
    u_int16_t   maxW;
    u_int16_t   maxK;
    u_int16_t   decend;
    u_int16_t   rectW;
    u_int16_t   rectH;
    u_int16_t   tableW;
    u_int16_t   maxA;
    u_int16_t   maxD;
    u_int16_t   leading;
    u_int16_t   imageW;
} nfntHeader;



// width and offset will contain FF if there is no glyph
typedef struct
{
    u_int8_t offset;  // offset into bitmap to glyph
    u_int8_t width;   // width of the bitmap
} nfntOffsetWidthT;


typedef u_int16_t nfntLocationT;


void showline(char line)
{
    if (line & 0x80) printf("#"); else printf(".");
    if (line & 0x40) printf("#"); else printf(".");
    if (line & 0x20) printf("#"); else printf(".");
    if (line & 0x10) printf("#"); else printf(".");
    if (line & 0x08) printf("#"); else printf(".");
    if (line & 0x04) printf("#"); else printf(".");
    if (line & 0x02) printf("#"); else printf(".");
    if (line & 0x01) printf("#"); else printf(".");
}


int
main(int argc, char *argv[])
{
    char cyfontname[44];    // big enough...
    char *fontname;
    char *fonttype;
    int i;
    FILE *font;
    int  cyf;
    unsigned short stride;
    unsigned int FileOffsetWidth;
    unsigned int FileImage;
    unsigned int FontImageSize;
    unsigned int FontLocationSize;

    char *fontImage;    // malloc the memory for this

    // these arrays will be big enough
    nfntOffsetWidthT    fontOffsetWidth[256];
    nfntLocationT       fontLocation[256];

    BitmapSequenceT cyfontheader;


    // must have a font on the command line
    if (argc < 1)
    {
        printf("\nMust have fontname on command line!\n");
        return (1);
    }

    // open the font to read
    font = fopen(argv[1], "r");
    if (font == 0)
    {
        printf("\nCan't open font file to read!\n");
        return (1);
    }


    // convert the string to lowercase
    for (fontname = argv[1]; *fontname; fontname++)
    {
        *fontname = tolower(*fontname);
    }


    // find the last period so we can determine the file type
    // if there is no . or extention then we cant go on!
    fonttype = strrchr(argv[1], '.');
    if (!fonttype || (*++fonttype == 0))
    {
        printf("\nCan't determine filetype!\n");
        fclose(font);
        return (1);
    }


    // copy the full name in
    strcpy(cyfontname, argv[1]);

    // find the first dot
    fonttype = strchr(cyfontname, '.');

    // copy the extention into the first dot
    strcpy(fonttype, ".cyf");

    // open/clear/create the font to write - BINARY mode
    printf("filename %s\n", cyfontname);

    cyf = open(cyfontname,  O_RDWR | O_TRUNC | O_BINARY | O_CREAT, S_IRWXU);
    if (cyf == -1)
    {
        printf("\nError Creating/Opening File!\n");
        fclose(font);
        return 1;
    }

#define PDB_OFFSET  0x58

    /////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////  N F N T
    //////////////////////////////////////////////////////   Conversion

    // skip the pdb header - the font header starts after the file header
    fseek(font, PDB_OFFSET, SEEK_SET);

    // read the whole header at once
    fread(&nfntHeader, sizeof(nfntHeader), 1, font);

    // all of the shorts may be scrambled because of big and little endian
    for (i=0; i<sizeof(nfntHeader)/sizeof(u_int16_t); i++)
    {
        u_int16_t *ptr = ((u_int16_t *)&nfntHeader) + i;

        *ptr = ntohs(*ptr);
    }

    // now the shorts are in the order that this prog can deal with

    printf("font type: %hx\n", nfntHeader.type);
    printf("font first: %d\n", nfntHeader.first);
    printf("font last: %d\n", nfntHeader.last);
    printf("font maxW: %d\n", nfntHeader.maxW);
    printf("font maxK: %d\n", nfntHeader.maxK);
    printf("font decend: %d\n", nfntHeader.decend);
    printf("font rectW: %d\n", nfntHeader.rectW);
    printf("font rectH: %d\n", nfntHeader.rectH);
    printf("font tableW: %d\n", nfntHeader.tableW);
    printf("font maxA: %d\n", nfntHeader.maxA);
    printf("font maxD: %d\n", nfntHeader.maxD);
    printf("font leading: %d\n", nfntHeader.leading);
    printf("font imageW: %d\n", nfntHeader.imageW);


    if (nfntHeader.type != 0x9000)
    {
        printf("\n!! NOT a nftn type font!!\n ");
        fclose(font);
        return (1);
    }

    // anything lower than a space? cant use it
    if (nfntHeader.last < 0x20)
    {
        printf("\n!! NO usable characters!!\n ");
        fclose(font);
        return (1);
    }


    // The stride is how many bytes wide is one row of the font image
    // imageW is how many words
    stride = nfntHeader.imageW * 2;

    printf("Stride is %x\n", stride);

    // calculate the details for the file and the image
    // The width offset table is at:
    FileOffsetWidth = PDB_OFFSET + 0x10 + nfntHeader.tableW * 2;
    printf("FileOffsetWidth is %x\n", FileOffsetWidth);

    // location of the image in the pdb file
    // It follows the PDB header and fontheader
    FileImage = PDB_OFFSET + sizeof(nfntHeader);
    printf("FontImage start in file is %x\n", FileImage);

    // number of bytes in the font image = stride * height
    FontImageSize = stride * nfntHeader.rectH;
    printf("FontImage Size is %x\n", FontImageSize);

    // How many entries in location array
    FontLocationSize = (nfntHeader.last - nfntHeader.first) + 1 + 1;
    printf("FontLocationSize is %x\n", FontLocationSize);

    fontImage = (char *)malloc(FontImageSize);
    if (fontImage == 0)
    {
       printf("\n!! Cannot allocate memory!\n");
       fclose(font);
       return 1;
    }

    // get fontImage, fontLocation, fontWidthOffset
    fseek(font, FileImage, SEEK_SET);
    fread(fontImage, FontImageSize, 1, font);
    fread(fontLocation, sizeof(nfntLocationT), FontLocationSize, font);

    fseek(font, FileOffsetWidth, SEEK_SET);
    fread(fontOffsetWidth, sizeof(nfntOffsetWidthT), FontLocationSize-1, font);







//////////////////////  C O N V E R T   To Cybiko


    // figure and write out the header for the bitmapfont
    //cyfontheader.Type = CYBIKO_FILE_TYPE_FONT;
    cyfontheader.Type = 1;
    cyfontheader.Wmax = (char)nfntHeader.rectW;
    cyfontheader.Hmax = (char)nfntHeader.rectH;

    cyfontheader.Count = (char)nfntHeader.last - (char)nfntHeader.first + 1;

    // ignore/subtract the chars before 20
    if (nfntHeader.first < 0x20)
    {
        cyfontheader.Count -= (0x20 - nfntHeader.first);
    }

    // ignore/subtract the chars after 127
    if (nfntHeader.last > 127)
    {
        cyfontheader.Count -= (nfntHeader.last - 127);
    }

    // write the header to the new file
    write(cyf, &cyfontheader, sizeof(cyfontheader));

    // now convert everything
    for (i=0; i<cyfontheader.Count; i++)
    {
        char character;
        u_int16_t charfrom;
        u_int16_t locationFrom, locationTo, xFrom;
        BitmapHeaderT header;
        unsigned char bitline;
        u_int16_t x, y;

        // The cybiko font MUST start at 0x20 (space)
        // AND include EVERY bitmap inbetween. Even for missing glyphs
        character = i + ' ';
        charfrom  = character - nfntHeader.first;


        // get the pixel starting offset into the font image
        locationFrom = ntohs(fontLocation[charfrom]);

        // get the pixel starting offset into the NEXT font image
        locationTo = ntohs(fontLocation[charfrom+1]);


        // convert if in the right range and has an entry in the bitmap
        // otherwise output an empty bitmap
        if (character >= nfntHeader.first &&
            character <= nfntHeader.last &&
            (fontOffsetWidth[charfrom].offset != 0xFF) &&
            (fontOffsetWidth[charfrom].width != 0xFF) &&
            (locationFrom != locationTo))
        {
            // fill in the entire bitmap header and write it out
            header.Hoff = fontOffsetWidth[charfrom].offset;
            header.Wide = fontOffsetWidth[charfrom].width;
            header.Voff = 0;
            header.High = nfntHeader.rectH;
            write(cyf, &header, sizeof(header));

            // one font image raster line at a time
            for (y = 0; y < nfntHeader.rectH; y++)
            {
                x = 0;
                xFrom = locationFrom;
                bitline = 0;

                // do all the bits across
                for (;;)
                {
                    // check the font for a set bit
                    if (IS_FONT_SET(fontImage, xFrom, y, stride))
                    {
                        bitline |= (0x80 >> (x&7));
                    }

                    // bump the indexes
                    x++;
                    xFrom++;

                    // finished with a byte? or end of width?
                    if (((x & 7) == 0) || (xFrom == locationTo))
                    {
                        write(cyf, &bitline, 1);
                        if (xFrom == locationTo)
                        {
                            // next line
                            break;
                        }
                    }
                }
            }
        }
        else
        {
            // stick in an empty bitmap
            write(cyf, &EmptyBitmap, sizeof(EmptyBitmap));
        }
    }

    free(fontImage);
    close(cyf);
    fclose(font);

    return 0;
}



