#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#define IS_FONT_SET(image, x, y, stride) \
    (image[((x)>>3) + (y) * (stride)] & (0x80 >> ((x)&7)))


// header for each and every bitmap
typedef struct
{
    unsigned char Hoff; // horizontal offset
    unsigned char Voff; // vertical offset
    unsigned char Wide; // how wide is the bitmap
    unsigned char High; // how high
} BitmapHeaderT;



// found at the head of the file
typedef struct
{
    unsigned char   Type;       // what have we got in here?
    unsigned char   Count;      // how many bitmaps?
    unsigned char   Wmax;       // widest bitmap
    unsigned char   Hmax;       // highest bitmap
} BitmapSequenceT;


struct
{
    BitmapHeaderT   header;
    char            empty;
} EmptyBitmap = { 0, 0, 1, 1, 0 };



// convert font to cyborn / cybiko format


// NFNT format
// FON/FNT format


struct
{
    unsigned short type;
    unsigned short first;
    unsigned short last;
    unsigned short maxW;
    unsigned short maxK;
    unsigned short decend;
    unsigned short rectW;
    unsigned short rectH;
    unsigned short tableW;
    unsigned short maxA;
    unsigned short maxD;
    unsigned short leading;
    unsigned short imageW;
} nfntHeader;



// width and offset will contain FF if there is no glyph
typedef struct
{
    unsigned char offset;  // offset into
    unsigned char width;   // width of the bitmap
} nfntOffsetWidthT;


typedef struct
{
    unsigned char msb;      // use this because file is big endian
    unsigned char lsb;      // mult the msb then add lsb for intel
} nfntLocationT;


void showline(char line)
{
    if (line & 0x80) printf("#"); else printf(".");
    if (line & 0x40) printf("#"); else printf(".");
    if (line & 0x20) printf("#"); else printf(".");
    if (line & 0x10) printf("#"); else printf(".");
    if (line & 0x08) printf("#"); else printf(".");
    if (line & 0x04) printf("#"); else printf(".");
    if (line & 0x02) printf("#"); else printf(".");
    if (line & 0x01) printf("#"); else printf(".");
}


main(int argc, char *argv[])
{
    char cyfontname[44];    // big enough...
    char *fontname;
    char *fonttype;
    int i, j;
    FILE *font;
    int  cyf;
    unsigned int face;
    char face1;
    char face2;
    char face3;
    char face4;
    short start;
    unsigned short stride;
    unsigned int FileOffsetWidth;
    unsigned int FileImage;
    unsigned int FontImageSize;
    unsigned int FontLocationSize;

    char *fontImage;    // malloc the memory for this

    // these arrays will be big enough
    nfntOffsetWidthT    fontOffsetWidth[256];
    nfntLocationT       fontLocation[256];

    BitmapSequenceT cyfontheader;


    // must have a font on the command line
    if (argc < 1)
    {
        printf("\nMust have fontname on command line!\n");
        return (1);
    }

    // open the font to read
    font = fopen(argv[1], "r");
    if (font == 0)
    {
        printf("\nCan't open font file to read!\n");
        return (1);
    }


    // convert the string to lowercase
    for (fontname = argv[1]; *fontname; fontname++)
    {
        *fontname = tolower(*fontname);
    }


    // find the last period so we can determine the file type
    // if there is no . or extention then we cant go on!
    fonttype = strrchr(argv[1], '.');
    if (!fonttype || (*++fonttype == 0))
    {
        printf("\nCan't determine filetype!\n");
        fclose(font);
        return (1);
    }


    // copy the full name in
    strcpy(cyfontname, argv[1]);

    // find the first dot
    fonttype = strchr(cyfontname, '.');

    // copy the extention into the first dot
    strcpy(fonttype, ".cyf");

    // open/clear/create the font to write - BINARY mode
    printf("filename %s\n", cyfontname);

    cyf = open(cyfontname,  O_RDWR | O_TRUNC | O_BINARY | O_CREAT, S_IRWXU);
    if (cyf == -1)
    {
        printf("\nError Creating/Opening File!\n");
        fclose(font);
        return 1;
    }


    /////////////////////////////////////////////////////////////////////
    ///////////////////////////////////////////////////////  N F N T
    //////////////////////////////////////////////////////   Conversion



    // skip the pdb header - the font header starts after
    fseek(font, 0x58, SEEK_SET);

    fread(&nfntHeader, sizeof(nfntHeader), 1, font);


    // all of the shorts are scrambled because they are in big endian!
    // swap msb and lsb for each short (16 bits)
    for (i=0; i<sizeof(nfntHeader)/sizeof(short); i++)
    {
        short *ptr = ((short *)&nfntHeader) + i;

        *ptr = (*ptr>>8 & 0xFF) | (*ptr<<8 & 0xFF00);
    }

    // now the shorts are in the order that intel can deal with

    printf("font type: %hx\n", nfntHeader.type);
    printf("font first: %d\n", nfntHeader.first);
    printf("font last: %d\n", nfntHeader.last);
    printf("font maxW: %d\n", nfntHeader.maxW);
    printf("font maxK: %d\n", nfntHeader.maxK);
    printf("font decend: %d\n", nfntHeader.decend);
    printf("font rectW: %d\n", nfntHeader.rectW);
    printf("font rectH: %d\n", nfntHeader.rectH);
    printf("font tableW: %d\n", nfntHeader.tableW);
    printf("font maxA: %d\n", nfntHeader.maxA);
    printf("font maxD: %d\n", nfntHeader.maxD);
    printf("font leading: %d\n", nfntHeader.leading);
    printf("font imageW: %d\n", nfntHeader.imageW);


    if (nfntHeader.type != 0x9000)
    {
        printf("\n!! NOT a nftn type font!!\n ");
        fclose(font);
        return (1);
    }

    if (nfntHeader.last < 0x20)
    {
        printf("\n!! NO usable characters!!\n ");
        fclose(font);
        return (1);
    }


    // The stride is how many bytes wide is one row of the font image
    stride = nfntHeader.imageW * 2;

printf("Stride is %x\n", stride);

    // calculate the details for the file and the image
    // width offset table is at
    FileOffsetWidth = 0x58 + 0x10 + nfntHeader.tableW * 2;
printf("FileOffsetWidth is %x\n", FileOffsetWidth);

    // location of the image in the pdb file
    FileImage = 0x58 + sizeof(nfntHeader);
printf("FileImage is %x\n", FileImage);

    // number of bytes in the font image
    FontImageSize = stride * nfntHeader.rectH;
printf("FontImageSize is %x\n", FontImageSize);

    // How many entries in location array
    FontLocationSize = (nfntHeader.last - nfntHeader.first) + 1 + 1;
printf("FontLocationSize is %x\n", FontLocationSize);

    fontImage = (char *)malloc(FontImageSize);
    if (fontImage == 0)
    {
       printf("\n!! Cannot allocate memory!\n");
       fclose(font);
       return 1;
    }

    // get fontImage, fontLocation, fontWidthOffset
    fseek(font, FileImage, SEEK_SET);
    fread(fontImage, FontImageSize, 1, font);
    fread(fontLocation, sizeof(nfntLocationT), FontLocationSize, font);

    fseek(font, FileOffsetWidth, SEEK_SET);
    fread(fontOffsetWidth, sizeof(nfntOffsetWidthT), FontLocationSize-1, font);


    // figure and write out the header for the bitmapfont
    //cyfontheader.Type = CYBIKO_FILE_TYPE_FONT;
    cyfontheader.Type = 1;
    cyfontheader.Wmax = (char)nfntHeader.rectW;
    cyfontheader.Hmax = (char)nfntHeader.rectH;

    cyfontheader.Count = (char)nfntHeader.last - (char)nfntHeader.first + 1;

    // ignore/subtract the chars before 20
    if (nfntHeader.first < 0x20)
    {
        cyfontheader.Count -= (0x20 - nfntHeader.first);
    }

    // ignore/subtract the chars after 127
    if (nfntHeader.last > 127)
    {
        cyfontheader.Count -= (nfntHeader.last - 127);
    }

    // write the header to the new file
    write(cyf, &cyfontheader, sizeof(cyfontheader));

    // now convert everything
    for (i=0; i<cyfontheader.Count; i++)
    {
        char character, charfrom;
        unsigned short locationFrom, locationTo, xFrom;
        BitmapHeaderT header;
        unsigned char bitline;
        unsigned short x, y;

        // The cybiko font MUST start at 0x20 (space)
        // AND include EVERY bitmap inbetween. Even for missing glyphs
        character = i + ' ';
        charfrom  = character - nfntHeader.first;


        // get the pixel starting offset into the font image
        locationFrom = fontLocation[charfrom].lsb +
            ((int)fontLocation[charfrom].msb << 8);

        // get the pixel starting offset into the NEXT font image
        locationTo = fontLocation[charfrom+1].lsb +
                ((int)fontLocation[charfrom+1].msb << 8);


        // convert if in the right range and has an entry in the bitmap
        // otherwise output an empty bitmap
        if (character >= nfntHeader.first &&
            character <= nfntHeader.last &&
            (fontOffsetWidth[charfrom].offset != 0xFF) &&
            (fontOffsetWidth[charfrom].width != 0xFF) &&
            (locationFrom != locationTo))
        {
            // fill in the entire bitmap header and write it out
            header.Hoff = fontOffsetWidth[charfrom].offset;
            header.Wide = fontOffsetWidth[charfrom].width;
            header.Voff = 0;
            header.High = nfntHeader.rectH;
            write(cyf, &header, sizeof(header));

            // one font image raster line at a time
            for (y = 0; y < nfntHeader.rectH; y++)
            {
                x = 0;
                xFrom = locationFrom;
                bitline = 0;

                // do all the bits across
                for (;;)
                {
                    // check the font for a set bit
                    if (IS_FONT_SET(fontImage, xFrom, y, stride))
                    {
                        bitline |= (0x80 >> (x&7));
                    }

                    // bump the indexes
                    x++;
                    xFrom++;

                    // finished with a byte? or end of width?
                    if (((x & 7) == 0) || (xFrom == locationTo))
                    {
                        write(cyf, &bitline, 1);
                        if (xFrom == locationTo)
                        {
                            // next line
                            break;
                        }
                    }
                }
            }
        }
        else
        {
            // stick in an empty bitmap
            write(cyf, &EmptyBitmap, sizeof(EmptyBitmap));
        }
    }

    free(fontImage);
    close(cyf);
    fclose(font);
}



